#!/usr/bin/python2.5
#
# Copyright 2009 Olivier Gillet.
#
# Author: Olivier Gillet (ol.gillet@gmail.com)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -----------------------------------------------------------------------------
#
# Midifile Writer and Reader.

"""Midifile writer.
"""

import bisect
import math
import struct


def PackInteger(value, size=4):
  """Packs a python integer into a n-byte big endian byte sequence."""
  return struct.pack('>%s' % {1: 'B', 2: 'H', 4: 'L'}[size], value)


def UnpackInteger(value, size=4):
  """Packs a python integer into a n-byte big endian byte sequence."""
  return struct.unpack('>%s' % {1: 'B', 2: 'H', 4: 'L'}[size], value)[0]


def PackVariableLengthInteger(value):
  """Packs a python integer into a variable length byte sequence."""
  if value == 0:
    return '\x00'
  s = value
  output = []
  while value:
    to_write = value & 0x7f
    value = value >> 7
    output.insert(0, to_write)
  for i in xrange(len(output) - 1):
    output[i] |= 0x80
  output = ''.join(map(chr, output))
  return output

"""Classes representing a MIDI event, with a Serialize method which encodes
them into a string."""

class Event(object):
  def __init__(self):
    pass
    
  def Serialize(self, running_status):
    raise NotImplementedError


class MetaEvent(Event):
  def __init__(self, id, data):
    assert len(data) < 256
    self.id = id
    self.data = data
    
  def Serialize(self, running_status):
    return ''.join([
        '\xff',
        PackInteger(self.id, size=1),
        PackInteger(len(self.data), size=1),
        self.data]), None


class TextEvent(MetaEvent):
  def __init__(self, text):
    self.text = text
    super(TextEvent, self).__init__(0x01, text)


class CopyrightInfoEvent(MetaEvent):
  def __init__(self, text):
    self.text = text
    super(CopyrightInfoEvent, self).__init__(0x02, text)


class TrackNameEvent(MetaEvent):
  def __init__(self, text):
    self.text = text
    super(TrackNameEvent, self).__init__(0x03, text)


class TrackInstrumentNameEvent(MetaEvent):
  def __init__(self, text):
    self.text = text
    super(TrackInstrumentNameEvent, self).__init__(0x04, text)


class LyricEvent(MetaEvent):
  def __init__(self, text):
    self.text = text
    super(LyricEvent, self).__init__(0x05, text)


class MarkerEvent(MetaEvent):
  def __init__(self, text):
    self.text = text
    super(MarkerEvent, self).__init__(0x06, text)


class CuePointEvent(MetaEvent):
  def __init__(self, text):
    self.text = text
    super(CuePointEvent, self).__init__(0x07, text)


class EndOfTrackEvent(MetaEvent):
  def __init__(self):
    super(EndOfTrackEvent, self).__init__(0x2f, '')


class TempoEvent(MetaEvent):
  def __init__(self, bpm):
    self.bpm = bpm
    value = 60000000.0 / bpm
    data = PackInteger(int(value), size=4)[1:]
    super(TempoEvent, self).__init__(0x51, data)


class SMPTEOffsetEvent(MetaEvent):
  def __init__(self, h, m, s, f, sf):
    self.smpte_offset = (h, m, s, f, sf)
    data = ''.join(map(chr, [h, m, s, f, sf]))
    super(SMPTEOffsetEvent, self).__init__(0x54, data)


class TimeSignatureEvent(MetaEvent):
  def __init__(self, numerator, denominator):
    self.numerator = numerator
    self.denominator = denominator
    data = ''.join([
        PackInteger(numerator, size=1),
        PackInteger(int(math.log(denominator) / math.log(2)), size=1),
        '\x16\x08'])
    super(TimeSignatureEvent, self).__init__(0x58, data)


class KeyEvent(MetaEvent):
  def __init__(self, sharp_flats, major_minor):
    self.sharp_flats = sharp_flats
    self.major_minor = major_minor
    data = ''.join([
        PackInteger(sharp_flats, size=1),
        PackInteger(major_minor, size=1)])
    super(KeyEvent, self).__init__(0x59, data)


class BlobEvent(MetaEvent):
  def __init__(self, blob):
    self.data = blob
    super(BlobEvent, self).__init__(0x7f, blob)


"""Classic channel-oriented messages."""

class ChannelEvent(Event):
  def __init__(self, mask, channel, data):
    self.channel = channel
    self._status = mask | (channel - 1)
    self._data = PackInteger(self._status, size=1) + data
    
  def Serialize(self, running_status):
    if self._status == running_status:
      return self._data[1:], self._status
    else:
      return self._data, self._status


class NoteOffEvent(ChannelEvent):
  def __init__(self, channel, note, velocity):
    data = PackInteger(note, size=1) + PackInteger(velocity, size=1)
    super(NoteOffEvent, self).__init__(0x80, channel, data)
    self.note = note
    self.velocity = velocity


class NoteOnEvent(ChannelEvent):
  def __init__(self, channel, note, velocity):
    data = PackInteger(note, size=1) + PackInteger(velocity, size=1)
    super(NoteOnEvent, self).__init__(0x90, channel, data)
    self.note = note
    self.velocity = velocity


class KeyAftertouchEvent(ChannelEvent):
  def __init__(self, channel, note, aftertouch):
    data = PackInteger(note, size=1) + PackInteger(aftertouch, size=1)
    super(KeyAftertouchEvent, self).__init__(0xa0, channel, data)
    self.note = note
    self.aftertouch = aftertouch


class ControlChangeEvent(ChannelEvent):
  def __init__(self, channel, controller, value):
    data = PackInteger(controller, size=1) + PackInteger(value, size=1)
    super(ControlChangeEvent, self).__init__(0xb0, channel, data)
    self.controller = controller
    self.value = value


class ProgramChangeEvent(ChannelEvent):
  def __init__(self, channel, program_number):
    data = PackInteger(program_number, size=1)
    super(ProgramChangeEvent, self).__init__(0xc0, channel, data)
    self.program_number = program_number


class ChannelAftertouchEvent(ChannelEvent):
  def __init__(self, channel, aftertouch):
    data = PackInteger(aftertouch, size=1)
    super(ChannelAftertouchEvent, self).__init__(0xd0, channel, data)
    self.aftertouch = aftertouch


class PitchBendEvent(ChannelEvent):
  def __init__(self, channel, pitch_bend_14_bits):
    data = PackInteger(pitch_bend_14_bits >> 7, size=1) + \
        PackInteger(pitch_bend_14_bits & 0x7f, size=1)
    super(PitchBendEvent, self).__init__(0xe0, channel, data)
    self.pitch_bend = pitch_bend_14_bits


class SystemEvent(Event):
  def __init__(self, id):
    self._id = id
    
  def Serialize(self, running_status):
    return PackInteger(self._id, size=1), running_status


class ClockEvent(SystemEvent):
  def __init__(self):
    super(ClockEvent, self).__init__(0xf8)


class StartEvent(SystemEvent):
  def __init__(self):
    super(StartEvent, self).__init__(0xfa)


class ContinueEvent(SystemEvent):
  def __init__(self):
    super(ContinueEvent, self).__init__(0xfb)


class StopEvent(SystemEvent):
  def __init__(self):
    super(StopEvent, self).__init__(0xfc)


# TODO(pichenettes): also support pauses within a block transmission (F7)
class SysExEvent(Event):
  def __init__(self, manufacturer_id, device_id, data):
    self.data = data
    self.message = ''.join([
        manufacturer_id,
        device_id,
        data,
        '\xf7'])
    self.raw_message = '\xf0' + self.message
    assert all(ord(x) < 128 for x in self.message[:-1])
    self.message = ''.join([
        '\xf0',
        PackVariableLengthInteger(len(self.message)),
        self.message])

  def Serialize(self, running_status):
    return self.message, None


def Nibblize(data, add_checksum=True):
  """Converts a byte string into a nibble string. Also adds checksum"""
  output = []
  if add_checksum:
    tail = [chr(sum(ord(char) for char in data) % 256)]
  else:
    tail = []
  for char in map(ord, list(data) + tail):
    output.append(chr(char >> 4))
    output.append(chr(char & 0x0f))
  return ''.join(output)


class Track(object):
  def __init__(self):
    self._events = []
    
  def AddEvent(self, time, event):
    self._events.append((time, event))

  def Sort(self):
    self._events = sorted(self._events)

  def Serialize(self):
    self.Sort()
    last_time, last_event = self._events[-1]
    if type(last_event) != EndOfTrackEvent:
      self._events.append((last_time + 1, EndOfTrackEvent()))
    data = []
    current_time = 0
    running_status = None
    for time, event in self._events:
      delta = time - current_time
      data.append(PackVariableLengthInteger(delta))
      event_data, running_status = event.Serialize(running_status)
      data.append(event_data)
      current_time = time
    return ''.join(data)
    
  def Write(self, file_object):
    file_object.write('MTrk')
    track_data = self.Serialize()
    file_object.write(PackInteger(len(track_data)))
    file_object.write(track_data)
    
  @property
  def events(self):
    return self._events

    
class Writer(object):
  def __init__(self, ppq=96):
    self._tracks = []
    self._ppq = ppq
  
  def AddTrack(self):
    new_track = Track()
    self._tracks.append(new_track)
    return new_track
    
  def _MergeTracks(self):
    new_track = Track()
    for track in self._tracks:
      for time_event in track.events:
        new_track.AddEvent(*time_event)
    new_track.Sort()
    return new_track
    
  def Write(self, file_object, format=0):
    tracks = self._tracks
    if format == 0:
      tracks = [self._MergeTracks()]

    # File header.
    file_object.write('MThd')
    file_object.write(PackInteger(6))
    file_object.write(PackInteger(format, size=2))
    if format == 0:
      file_object.write(PackInteger(1, size=2))
    else:
      file_object.write(PackInteger(len(self._tracks), size=2))
    file_object.write(PackInteger(self._ppq, size=2))
    
    # Tracks.
    for track in tracks:
      track.Write(file_object)


class Reader(object):
  def __init__(self):
    self.tracks = []
    self.format = 0
    self.ppq = 96
    self._previous_status = 0
    
  def Read(self, f):
    assert f.read(4) == 'MThd'
    assert struct.unpack('>i', f.read(4))[0] == 6
    self.format = struct.unpack('>h', f.read(2))[0]
    assert self.format <= 2
    num_tracks = struct.unpack('>h', f.read(2))[0]
    self.ppq = struct.unpack('>h', f.read(2))[0]
    self._tempo_map = []
    
    for i in xrange(num_tracks):
      self.tracks.append(self._ReadTrack(f))
    self._CreateCumulativeTempoMap()
    
      
  def _ReadTrack(self, f):
    assert f.read(4) == 'MTrk'
    size = struct.unpack('>i', f.read(4))[0]
    t = 0
    events = []
    while size > 0:
      delta_t, e, event_size = self._ReadEvent(f)
      t += delta_t
      if e:
        events.append((t, e))
        if type(e) == TempoEvent:
          self._tempo_map.append((t, e.bpm))
      size -= event_size
    return events
    
  def _CreateCumulativeTempoMap(self):
    t = 0.0
    current_tempo = 120.0
    previous_beat = 0
    cumulative_tempo_map = [(0, 0.0, current_tempo)]
    for beat, tempo in sorted(self._tempo_map):
      beats = float(beat - previous_beat) / self.ppq
      t += beats * 60.0 / current_tempo
      cumulative_tempo_map.append((beat, t, tempo))
      current_tempo = tempo
      previous_beat = beat
    self._tempo_map = cumulative_tempo_map
    
  def AbsoluteTime(self, t):
    index = bisect.bisect_left(self._tempo_map, (t, 0, 0))
    index = max(index - 1, 0)
    start_beat, start_seconds, tempo = self._tempo_map[index]
    return start_seconds + float(t - start_beat) / self.ppq * 60.0 / tempo
    
  def _ReadVariableLengthInteger(self, f):
    v = 0
    size = 0
    while True:
      v <<= 7
      byte = UnpackInteger(f.read(1), size=1)
      size += 1
      v |= (byte & 0x7f)
      if not (byte & 0x80):
        break
    return v, size
    
  def _ReadEvent(self, f):
    delta_t, size = self._ReadVariableLengthInteger(f)
    event_byte = ord(f.read(1))
    size += 1
    if event_byte < 0x80:
      if self._previous_status:
        f.seek(f.tell() - 1)
        size -= 1
        event_byte = self._previous_status
      else:
        return delta_t, None, size
    
    event_type = event_byte & 0xf0
    channel = event_byte & 0xf
    channel += 1
    if event_type == 0x80:
      self._previous_status = event_type
      note = ord(f.read(1))
      velo = ord(f.read(1))
      event = NoteOffEvent(channel, note, velo)
      size += 2
    elif event_type == 0x90:
      self._previous_status = event_type
      event = NoteOnEvent(channel, ord(f.read(1)), ord(f.read(1)))
      size += 2
    elif event_type == 0xa0:
      self._previous_status = event_type
      event = KeyAftertouchEvent(channel, ord(f.read(1)), ord(f.read(1)))
      size += 2
    elif event_type == 0xb0:
      self._previous_status = event_type
      event = ControlChangeEvent(channel, ord(f.read(1)), ord(f.read(1)))
      size += 2
    elif event_type == 0xc0:
      self._previous_status = event_type
      event = ProgramChangeEvent(channel, ord(f.read(1)))
      size += 1
    elif event_type == 0xd0:
      self._previous_status = event_type
      event = ChannelAftertouchEvent(channel, ord(f.read(1)))
      size += 1
    elif event_type == 0xe0:
      self._previous_status = event_type
      event = PitchBendEvent(channel, (ord(f.read(1)) << 7) | ord(f.read(1)))
      size += 2
    elif event_byte == 0xff:
      event_type = ord(f.read(1))
      size += 1
      event_size, event_size_size = self._ReadVariableLengthInteger(f)
      size += event_size_size
      bytes = f.read(event_size)
      size += event_size
      if event_type == 0x01:
        event = TextEvent(bytes)
      elif event_type == 0x02:
        event = CopyrightInfoEvent(bytes)
      elif event_type == 0x03:
        event = TrackNameEvent(bytes)
      elif event_type == 0x04:
        event = TrackInstrumentNameEvent(bytes)
      elif event_type == 0x05:
        event = LyricEvent(bytes)
      elif event_type == 0x06:
        event = MarkerEvent(bytes)
      elif event_type == 0x07:
        event = CuePointEvent(bytes)
      elif event_type == 0x20:
        current_channel = ord(bytes[0])
        event = None
      elif event_type == 0x2f:
        event = EndOfTrackEvent()
      elif event_type == 0x51:
        value = UnpackInteger('\x00' + bytes, size=4)
        event = TempoEvent(60000000.0 / value)
      elif event_type == 0x54:
        event = SMPTEOffsetEvent(*map(ord, bytes))
      elif event_type == 0x58:
        event = TimeSignatureEvent(ord(bytes[0]), 2 ** ord(bytes[1]))
      elif event_type == 0x59:
        event = KeyEvent(ord(bytes[0]), ord(bytes[1]))
      elif event_type == 0x7f:
        event = BlobEvent(bytes)
    elif event_byte == 0xf0:
      event_size, event_size_size = self._ReadVariableLengthInteger(f)
      size += event_size_size
      bytes = f.read(event_size)
      size += event_size
      event = SysExEvent(bytes[0:3], bytes[3:5], bytes[5:-1])
    else:
      print event_byte, '!!'
      event = None
    return delta_t, event, size
    

if __name__ == '__main__':
  m = Writer()
  t = m.AddTrack()
  t.AddEvent(0, TempoEvent(120.0))
  
  t = m.AddTrack()
  t.AddEvent(1, SysExEvent(
      '\x00\x20\x77',
      '\x00\x01',
      '\x7f\x7f' + Nibblize('\xff\x00\xcc')))
  
  f = file('output.mid', 'wb')
  m.Write(f, format=0)
  f.close()



import numpy
import pylab

def dumb(f):
  return f * numpy.pi

def pade(f):
  return f * -0.90585 / (-0.28833 + f * f)

def poly3taylor(f):
  fsq = f * f
  r = 0.3333333 * numpy.pi ** 3
  r *= fsq
  r += numpy.pi
  r *= f
  return r

def poly3gradient(f, a=3.736e-01):
  fsq = f * f
  r = a * numpy.pi ** 3
  r *= fsq
  r += numpy.pi
  r *= f
  return r


def poly5mdsp(f, a=3.1755e-01, b=2.033e-01):
  fsq = f * f
  r = b * numpy.pi ** 5
  r *= fsq
  r += a * numpy.pi ** 3
  r *= fsq
  r += numpy.pi
  r *= f
  return r


def poly5gradient(f, a=3.260e-01, b=1.823e-01):
  f = f * numpy.pi
  fsq = f * f
  r = b
  r *= fsq
  r += a
  r *= fsq
  r += 1.0
  r *= f
  return r


def poly11mdsp(f):
  fsq = f * f
  r = 9.5168091e-03 * numpy.pi ** 11
  r *= fsq
  r += 2.900525e-03 * numpy.pi ** 9
  r *= fsq
  r += 5.33740603e-02 * numpy.pi **7
  r *= fsq
  r += 1.333923995e-01 * numpy.pi **5
  r *= fsq
  r += 3.333314036e-01 * numpy.pi **3
  r *= fsq
  r += numpy.pi
  r *= f
  return r


def compute_filter_settings(cutoff, resonance):
  g = numpy.tan(numpy.pi * cutoff) + resonance * 0
  r = 1.0 / resonance + cutoff * 0
  h = 1 / (1 + r * g + g * g)
  return g, r, h


def evaluate(groundtruth_f, approximate_g):
  approximate_f = numpy.arctan(approximate_g) / numpy.pi
  return numpy.log2(approximate_f / groundtruth_f) * 1200.0

  
f = numpy.exp(numpy.linspace(numpy.log(16), numpy.log(10000), 1000.0))
f /= 48000.0
g, _, _ = compute_filter_settings(f, 0.5)
approximations = [pade, poly3gradient, poly5mdsp, poly5gradient, poly11mdsp]
#
# a_ = numpy.linspace(3.259e-01, 3.261e-01, 100)
# b_ = numpy.linspace(1.822e-01, 1.823e-01, 100)
# error = numpy.zeros((100, 100))
# best = 1e8
# arg_best = None
# for i, a in enumerate(a_):
#   for j, b in enumerate(b_):
#     error[i, j] = (evaluate(f, fast0(f, a, b)) ** 2).sum()
#     if error[i, j] < best:
#       best = error[i, j]
#       arg_best = (a, b)
#
# print arg_best
# pylab.plot(coefficient, error)
# pylab.show()

pylab.figure(figsize=(15,10))
for i, a in enumerate(approximations):
  n = len(approximations)
  # pylab.subplot(n * 100 + 10 + i + 1)
  pylab.plot(f * 48000, evaluate(f, a(f)))
  pylab.xlabel('Hz')
  pylab.ylabel('$\delta$ cents')

pylab.legend(map(lambda x: x.__name__, approximations))
pylab.tight_layout()
#pylab.savefig('plot.pdf')
pylab.show()



#!/usr/bin/python2.5
#
# Copyright 2012 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -----------------------------------------------------------------------------
#
# Master resources file.

header = """// Copyright 2012 Emilie Gillet.
//
// Author: Emilie Gillet (emilie.o.gillet@gmail.com)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// -----------------------------------------------------------------------------
//
// Resources definitions.
//
// Automatically generated with:
// make resources
"""

namespace = 'edges'
target = 'edges'
modifier = 'PROGMEM'
types = ['uint8_t', 'uint16_t']
includes = """
#include "avrlibx/avrlibx.h"

#include <avr/pgmspace.h>
"""

import lookup_tables
import waveforms

create_specialized_manager = True

resources = [
  ('dummy', 'string', 'STR_RES', 'prog_char', str, True),
  (lookup_tables.lookup_tables,
   'lookup_table', 'LUT_RES', 'prog_uint16_t', int, True),
  (lookup_tables.lookup_tables_32,
   'lookup_table_32', 'LUT_RES', 'prog_uint32_t', int, True),
  (waveforms.waveforms,
   'waveform', 'WAV_RES', 'prog_uint8_t', int, True),
]



#!/usr/bin/python2.5
#
# Copyright 2014 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without retri_stackction, including without limitation the rights
# to use, copy, modify, merge, publish, ditri_stackbute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
# See http://creativecommons.org/licenses/MIT/ for more information.
#
# -----------------------------------------------------------------------------
#
# Waveform definitions.

import numpy

WAVETABLE_SIZE = 257

wavetables = []

def scale(x):
  maximum = numpy.abs(x).max()
  return numpy.round(x / maximum * 32766.0)


def sine(frequency):
  t = numpy.arange(0, WAVETABLE_SIZE) / float(WAVETABLE_SIZE - 1)
  if frequency >= WAVETABLE_SIZE / 2:
    return t * 0
  t[-1] = t[0]
  x = numpy.sin(2 * numpy.pi * t * frequency)
  return x


def comb(n):
  x = 0
  for i in xrange(n):
    x += sine(i + 1)
  return x


def tri(n, f=1):
  x = 0
  for i in xrange(n):
    x += sine((2 * i + 1) * f) / (2 * i + 1) ** 2.0
  return x


def tri_stack_bright(n):
  x = 0
  for i in xrange(n):
    x += tri(15 + 5 * n, i + n / 3)
  return x


def tri_stack(n):
  x = 0
  for i in xrange(n):
    x += tri(5 + 7 * n, i + 1) / ((i + 1) ** 0.5)
  return x


def saw(n, f=1):
  x = 0
  for i in xrange(n):
    x += sine((i + 1) * f) / (i + 1)
  return x


def saw_stack(n):
  x = 0
  for i in xrange(n):
    x += saw(1 + 6 * i, i + 1) / ((i + 1) ** 0.5)
  return x


def square(n):
  x = 0
  for i in xrange(n):
    x += sine(2 * i + 1) / (2 * i + 1)
  return x


def quadra(n):
  x = 0
  for harmonic, amplitude in zip(xrange(4), [1, 0.5, 1, 0.5]):
    x += sine(2 * n + 2 * harmonic + 1) * amplitude
  return x


def drawbars(bars):
  pipes = [1.0, 3.0, 2.0, 4.0, 6.0, 8.0, 10.0, 12.0, 16.0]
  x = 0
  for intensity, frequency in zip(bars, pipes):
    x += int(intensity) / 8.0 * sine(frequency)
  return x


def pulse(duty):
  t = numpy.arange(0, WAVETABLE_SIZE) / float(WAVETABLE_SIZE - 1)
  t[-1] = t[0]
  t[t < duty] = -1.0
  t[t >= duty] = 1.0
  return -t


def burst(duty):
  t = numpy.arange(0, WAVETABLE_SIZE) / float(WAVETABLE_SIZE - 1)
  t[-1] = t[0]
  d = duty ** 0.5
  t[t < d] = -1.0
  t[t >= d] = 0.0
  return -t * sine(1.0 / duty)


def hybrid(duty):
  cycle = (numpy.arange(0, WAVETABLE_SIZE) + int((duty - 0.5) * WAVETABLE_SIZE)) % WAVETABLE_SIZE
  x = pulse(duty)
  x += saw(80)[cycle]
  x -= (x.mean())
  return x


def trisaw(harmonic):
  return tri(80) + saw(80, harmonic) * (1 if harmonic != 1 else 0) * 0.5


def square_formant(ratio):
  t = numpy.arange(0, WAVETABLE_SIZE) / float(WAVETABLE_SIZE - 1)
  phase = t * (ratio ** 0.5) * 0.5
  phase[phase >= 1.0] = 1.0
  amplitude = numpy.cos(phase * numpy.pi) + 1
  formant = (sine(ratio * 0.75) + 1.0) * amplitude * 0.5
  formant -= (formant.max() + formant.min()) / 2.0
  return formant
  

def saw_formant(ratio):
  t = numpy.arange(0, WAVETABLE_SIZE) / float(WAVETABLE_SIZE - 1)
  amplitude = 1.0 - t
  formant = (sine(ratio) + 1.0) * amplitude * 0.5
  formant -= (formant.max() + formant.min()) / 2.0
  return formant


def bandpass_formant(ratio):
  t = numpy.arange(0, WAVETABLE_SIZE) / float(WAVETABLE_SIZE - 1)
  amplitude = 1.0 - t
  formant = sine(ratio * 1.5) * amplitude * 0.5
  return formant


def sine_power(power):
  x = sine(1.0)
  if power >= 6:
    x = sine(2.0) * 2.0
  x += saw(16)
  power = 2.0 ** (1.2  * (power - 0.5))
  return numpy.sign(x) * (numpy.abs(x) ** power)


def formant_f(index):
  formant_1 = 3.9 * (index + 1) / 8.0
  formant_2 = (1.0 - numpy.cos(formant_1 * numpy.pi * 0.8))
  t = numpy.arange(0, WAVETABLE_SIZE) / float(WAVETABLE_SIZE - 1)
  amplitude_1 = (1.0 - t) ** 0.2 * numpy.exp(-4.0 * t) 
  amplitude_2 = (1.0 - t) ** 0.2 * numpy.exp(-2.0 * t) 
  formant_3 = sine(1 + 2.8 * (formant_2 + formant_1)) * amplitude_2 * 1.7
  formant_1 = sine(1 + 3 * formant_1) * amplitude_1
  formant_2 = sine(1 + 4 * formant_2) * amplitude_2 * 1.5
  f = formant_1 + formant_2 + formant_3
  return f - (f.max() + f.min()) / 2.0


def distort(x):
  return numpy.arctan(x * 6.0) / numpy.pi


def digi_formant_f(index):
  formant_1 = 3.8 * (index + 1) / 8.0
  formant_2 = (1.0 - numpy.cos(formant_1 * numpy.pi * 0.4))
  t = numpy.arange(0, WAVETABLE_SIZE) / float(WAVETABLE_SIZE - 1)
  amplitude_1 = (1.0 - t) ** 0.2 * numpy.exp(-4.0 * t) 
  amplitude_2 = (1.0 - t) ** 0.2 * numpy.exp(-2.0 * t) 
  formant_3 = distort(sine(1 + 2.9 * (formant_2 + formant_1))) * amplitude_2 * 0.7
  formant_1 = distort(sine(1 + 3.2 * formant_1)) * amplitude_1
  formant_2 = distort(sine(1 + 4.1 * formant_2)) * amplitude_2 * 0.7
  f = formant_1 + formant_2 + formant_3
  return f - (f.max() + f.min()) / 2.0


def make_bank(waveforms):
  bank = []
  for w in waveforms:
    bank += list(scale(w))
  return bank


def make_family(fn, arguments):
  return map(fn, arguments)

# Bank 1: mild and additive.
bank_1 = []
bank_1 += make_family(sine, [1, 2, 3, 4, 5, 6, 7, 8])
bank_1 += make_family(sine, [8, 10, 12, 14, 16, 18, 20, 22])
bank_1 += make_family(quadra, [1, 2, 3, 4, 5, 6, 7, 8])
bank_1 += make_family(comb, [2, 5, 9, 14, 20, 27, 35, 44])
bank_1 += make_family(tri_stack_bright, [2, 4, 6, 8, 10, 12, 14, 16])
bank_1 += make_family(tri_stack, [2, 3, 4, 5, 6, 7, 8, 9])
bank_1 += make_family(drawbars, [
    '800000888', '888000888', '867000006', '586040000',
    '850005000', '888643200', '327645222', '006050321'])
bank_1 += make_family(drawbars, [
    '680008880', '888876788', '848600046', '688600000',
    '666806000', '468844054', '004675300', '002478500',])

# Bank 2: formantish.
bank_2 = []
bank_2 += make_family(trisaw, [1, 1.5, 2, 3, 4, 4.5, 5, 8])
bank_2 += make_family(pulse, [0.5, 0.4, 1/3.0, 0.25, 0.2, 0.125, 1/16.0, 1/32.0])
bank_2 += make_family(burst, [0.5, 0.4, 1/3.0, 0.25, 0.2, 0.125, 1/16.0, 1/32.0])
bank_2 += make_family(square_formant, [2.0, 3.0, 4.0, 6.0, 8.0, 12.0, 16.0, 24.0])
bank_2 += make_family(bandpass_formant, [2.0, 3.0, 4.0, 6.0, 8.0, 12.0, 16.0, 32.0])
bank_2 += make_family(formant_f, xrange(8))
bank_2 += make_family(digi_formant_f, xrange(8))
bank_2 += make_family(sine_power, xrange(8))

# Bank 3: Shruthi.
bank_3 = list(numpy.fromstring(
    file('tides/resources/waves.bin', 'rb').read(), numpy.int16))

wavetables.append(('waves', make_bank(bank_1) + make_bank(bank_2) + bank_3[:50*257]))


"""----------------------------------------------------------------------------
Post waveshaper
----------------------------------------------------------------------------"""

WAVESHAPER_SIZE = 1024

waveshapers = []

x = numpy.arange(0, WAVESHAPER_SIZE + 1) / (WAVESHAPER_SIZE / 2.0) - 1.0
x[-1] = x[-2]
sine = numpy.sin(2 * numpy.pi * x)
window = numpy.exp(-x * x * 1.5) ** 2
bipolar_fold = sine * window + numpy.arctan(2 * x) * (1 - window)
bipolar_fold /= numpy.abs(bipolar_fold).max()
waveshapers.append(('smooth_bipolar_foldIN_RAM', numpy.round(32767 * bipolar_fold)))



#!/usr/bin/python2.5
#
# Copyright 2012 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -----------------------------------------------------------------------------
#
# Waveshaper lookup tables.

import numpy
import audio_io

boundaries = [0]
sample_data = []

TRUNCATE = False

for i in xrange(1, 10):
  audio_data, sr = audio_io.ReadWavFile('elements/samples/hit_%02d.wav' % i)
  audio_data = audio_data.sum(axis=1)
  audio_data = list(audio_data)
  if TRUNCATE:
    audio_data = audio_data[:128]
  audio_data += [audio_data[-1]]  # Add interpolation tail
  audio_data = numpy.round(numpy.array(audio_data) * 32767.0)
  sample_data += list(audio_data)
  boundaries.append(boundaries[-1] + len(audio_data))


sample_data = [('sample_data', sample_data)]
boundaries = [('boundaries', boundaries)]

audio_data, sr = audio_io.ReadWavFile('elements/samples/noise.wav')
audio_data = audio_data.sum(axis=1)
audio_data = numpy.round(numpy.array(audio_data) * 32767.0)
if TRUNCATE:
  audio_data = audio_data[:128]
sample_data += [('noise_sample', audio_data)]




#!/usr/bin/python2.5
#
# Copyright 2014 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
# See http://creativecommons.org/licenses/MIT/ for more information.
#
# -----------------------------------------------------------------------------
#
# Lookup table definitions.

import scipy.signal
import numpy
import pylab

lookup_tables = []
int16_lookup_tables = []

"""----------------------------------------------------------------------------
Cosine table.
----------------------------------------------------------------------------"""

size = 1024
t = numpy.arange(0, size + size / 4 + 1) / float(size) * numpy.pi * 2
lookup_tables.append(('sin', numpy.sin(t)))



"""----------------------------------------------------------------------------
Grain window.
----------------------------------------------------------------------------"""

size = 4096
t = numpy.arange(0, size + 1) / float(size)
lookup_tables.append(('window', 1.0 - (numpy.cos(t * numpy.pi) + 1) / 2))



"""----------------------------------------------------------------------------
XFade table
----------------------------------------------------------------------------"""

size = 17
t = numpy.arange(0, size) / float(size-1)
t = 1.04 * t - 0.02
t[t < 0] = 0
t[t >= 1] = 1
t *= numpy.pi / 2
lookup_tables.append(('xfade_in', numpy.sin(t) * 2 ** -0.5))
lookup_tables.append(('xfade_out', numpy.cos(t) * 2 ** -0.5))



"""----------------------------------------------------------------------------
Sine window.
----------------------------------------------------------------------------"""

def sum_window(window, steps):
  n = window.shape[0]
  start = 0
  stride = n / steps
  s = 0
  for i in xrange(steps):
    s = s + window[start:start+stride] ** 2
    start += stride
  return s


window_size = 4096
t = numpy.arange(0.0, window_size) / window_size

# Perfect reconstruction for overlap of 2
sine = numpy.sin(numpy.pi * t)

# Perfect reconstruction for overlap of 4
raised = (0.5 * numpy.cos(numpy.pi * t * 2) + 0.5) * numpy.sqrt(4.0 / 3.0)

# Needs tweaks to provide good reconstruction
power = (1.0 - (2 * t - 1.0) ** 2.0) ** 1.25
compensation = sum_window(power, 2) ** 0.5
compensation = numpy.array(list(compensation) * 2)
power /= compensation

lookup_tables.append(('sine_window_4096', power))



"""----------------------------------------------------------------------------
Linear to dB, for display
----------------------------------------------------------------------------"""

db = numpy.arange(0, 257)
db[0] = 1
db[db > 255] = 255
db = numpy.log2(db / 16.0) * 32768 / 4
int16_lookup_tables += [('db', db)]



"""----------------------------------------------------------------------------
LPG cutoff
----------------------------------------------------------------------------"""

TABLE_SIZE = 256

cutoff = numpy.arange(0.0, TABLE_SIZE + 1) / TABLE_SIZE
lookup_tables.append(('cutoff', 0.49 * 2 ** (-6 * (1 - cutoff))))



"""----------------------------------------------------------------------------
Grain size table
----------------------------------------------------------------------------"""

size = numpy.arange(0.0, TABLE_SIZE + 1) / TABLE_SIZE * 4
lookup_tables.append(('grain_size', numpy.floor(1024 * (2 ** size))))



"""----------------------------------------------------------------------------
Quantizer for pitch.
----------------------------------------------------------------------------"""

PITCH_TABLE_SIZE = 1025
pitch = numpy.zeros((PITCH_TABLE_SIZE, ))
notches = [-24, -12, -7, -4, -3, -1, -0.1, 0, 0, 0.1, 1, 3, 4, 7, 12, 24]
n = len(notches) - 1
for i in xrange(n):
  start_index = int(float(i) / n * PITCH_TABLE_SIZE)
  end_index = int(float(i + 1) / n * PITCH_TABLE_SIZE)
  length = end_index - start_index
  x = numpy.arange(0.0, length) / (length - 1)
  raised_cosine = 0.5 - 0.5 * numpy.cos(x * numpy.pi)
  xfade = 0.8 * raised_cosine + 0.2 * x
  pitch[start_index:end_index] = notches[i] + (notches[i + 1] - notches[i]) * xfade

lookup_tables.append(('quantized_pitch', pitch))



#!/usr/bin/python2.5
#
# Copyright 2014 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
# See http://creativecommons.org/licenses/MIT/ for more information.
#
# -----------------------------------------------------------------------------
#
# Lookup table definitions.

import numpy
import optparse
import os
import pylab
import scipy.signal
import sys



HEADER = """// Copyright 2015 Emilie Gillet.
//
// Author: Emilie Gillet (emilie.o.gillet@gmail.com)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// 
// See http://creativecommons.org/licenses/MIT/ for more information.
//
// -----------------------------------------------------------------------------
//
// Sample rate conversion filters.
"""



def plot_response(
    ir,
    ratio,
    name,
    n=8192,
    sr=96000.0,
    critical_frequency=16000.0):
  gain = 20 * numpy.log10(numpy.abs(numpy.fft.rfft(ir, n)) + 1e-30)
  f = numpy.arange(n / 2 + 1) / float(n) * sr * ratio
  bin_index = critical_frequency / ratio / sr * n
  critical_gain = gain[int(round(bin_index))]
  pylab.figure(figsize=(8, 12))
  pylab.subplot(211)
  pylab.plot(f, gain)
  pylab.xlim([0.0, sr])
  pylab.ylim([-80.0, 3.0])
  pylab.xlabel('Frequency (Hz)')
  pylab.ylabel('Gain (dB)')
  caption = 'Resampling filter, N=%d, Ratio=%d. Gain at %d Hz = %.2fdB'
  pylab.title(caption % (len(ir), ratio, critical_frequency, critical_gain))

  pylab.subplot(212)
  pylab.plot(f, gain)
  pylab.xlim([0.0, 20000.0])
  pylab.ylim([-3.0, 1.0])
  pylab.xlabel('Frequency (Hz)')
  pylab.ylabel('Gain (dB)')

  pylab.savefig(name + '.pdf')
  pylab.close()
  


def make_header_guard_name(name):
  name = name.upper()
  name = name.replace('/', '_')
  name = name.replace('.', '_')
  return name + '_' if name else ''
  


def generate_filters(options):
  output_file = options.output_file
  out = file(output_file, 'w') if output_file else sys.stderr
  guard = make_header_guard_name(output_file)
  namespace = output_file.split(os.sep)[0] if output_file else ''
  out.write(HEADER)
  
  if guard:
    out.write('\n#ifndef %s\n' % guard)
    out.write('#define %s\n' % guard)
  
  if namespace:
    out.write('\nnamespace %s {\n' % namespace)
  
  configurations = options.configurations.split(';')
  for configuration in configurations:
    tokens = configuration.split(',')
    ratio, length = map(int, tokens[:2])
    transition = float(tokens[2])
    ir = scipy.signal.remez(
        length,
        [0, transition / ratio, 0.5 / ratio, 0.5], [1, 0])
    ir /= sum(ir)
    cmd = 'scipy.signal.remez(%d, [0, %f / %d, 0.5 / %d, 0.5], [1, 0])' % \
        (length, transition, ratio, ratio)
    
    if options.plot_responses:
      name = 'src_filter_%d_%d' % (ratio, length)
      plot_response(ir, ratio, name)
    
    for up in [True, False]:
      scale = ratio if up else 1.0
      scaled = ir * scale
      out.write('\n// Generated with:\n')
      out.write('// %d * %s\n' % (scale, cmd))
      out.write('template<>\n')
      out.write('struct SRC_FIR<SRC_%s, %d, %d> {\n' % \
          ('UP' if up else 'DOWN', ratio, length))
      out.write('  template<int32_t i> inline float Read() const {\n')
      out.write('    const float h[] = {\n')
      for i in xrange(0, length / 2, 4):
        out.write('      ');
        out.write(', '.join(
            '% 16.9e' % scaled[j]  for j in xrange(i, min(length / 2, i + 4))))
        out.write(',\n');
      out.write('    };\n')
      out.write('    return h[i];\n')
      out.write('  }\n')
      out.write('};\n')
      
  if namespace:
    out.write('\n}  // namespace %s\n\n' % namespace)

  if guard:
    out.write('#endif  // %s\n' % guard)



if __name__ == '__main__':
  parser = optparse.OptionParser()
  parser.add_option(
      '-p',
      '--plot',
      dest='plot_responses',
      default=False,
      action='store_true',
      help='Plot responses')
  parser.add_option(
      '-c',
      '--configurations',
      dest='configurations',
      default='6,48,0.06;4,48,0.105;3,36,0.05',
      help='Output file name')
  parser.add_option(
      '-o',
      '--output',
      dest='output_file',
      default='warps/dsp/sample_rate_conversion_filters.h',
      help='Output file name')

  options, _ = parser.parse_args()
  generate_filters(options)



#!/usr/bin/python2.5
#
# Copyright 2012 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -----------------------------------------------------------------------------
#
# Lookup table definitions.

import numpy

lookup_tables = []
lookup_tables_32 = []

"""----------------------------------------------------------------------------
Timer oscillator pitch table.
----------------------------------------------------------------------------"""

notes = numpy.arange(0, 12 * 128 + 16, 16) / 128.0 + 24
frequencies = 2 ** ((notes - 69) / 12) * 440.0
values = 32000000.0 / (2 * 8 * frequencies)
lookup_tables.append(
    ('timer_count', numpy.round(values))
)

"""----------------------------------------------------------------------------
Digital oscillator pitch table.
----------------------------------------------------------------------------"""

sample_rate = 32000000 / 666.0
a4_midi = 69
a4_pitch = 440.0
excursion = 65536.0
notes = numpy.arange(116 * 128.0, 128 * 128.0 + 16, 16)
pitches = 2 * a4_pitch * 2 ** ((notes - a4_midi * 128) / (128 * 12))
increments = excursion / sample_rate * pitches

lookup_tables.append(
    ('oscillator_increments', increments.astype(int))
)

"""----------------------------------------------------------------------------
Bitcrusher pitch table.
----------------------------------------------------------------------------"""
ratios = [1] * 128
ratios = ratios + list(numpy.linspace(1.0, 64.0, 128))
increments = 65536 / numpy.array(ratios)
lookup_tables.append(
    ('bitcrusher_increments', increments.astype(int))
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -*- coding: utf-8 -*-
# vim: sw=4:ts=4:si:et:enc=utf-8

# Author: Ivan A-R <ivan@tuxotronic.org>
# Project page: http://tuxotronic.org/wiki/projects/stm32loader
# 
# This file is part of stm32loader.
# 
# stm32loader is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
# 
# stm32loader is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with stm32loader; see the file COPYING3.  If not see
# <http://www.gnu.org/licenses/>.

import sys, getopt
import serial
import time

try:
  from progressbar import *
  usepbar = 1
except:
  usepbar = 0

# Verbose level
QUIET = 20

def mdebug(level, message):
  if(QUIET >= level):
    print >> sys.stderr , message


class CmdException(Exception):
  pass

class CommandInterface:
  def open(self, aport='/dev/ttyUSB0', abaudrate=115200):
    self.sp = serial.Serial(
      port=aport,
      baudrate=abaudrate,   # baudrate
      bytesize=8,       # number of databits
      parity=serial.PARITY_EVEN,
      stopbits=1,
      xonxoff=0,        # enable software flow control
      rtscts=0,         # disable RTS/CTS flow control
      timeout=0.5         # set a timeout value, None for waiting forever
    )

  def _wait_for_ack(self, info="", timeout=0):
    stop = time.time() + timeout
    got = None
    while not got:
      got = self.sp.read(1)
      if time.time() > stop:
        break

    if not got:
      raise CmdException("No response to %s" % info)
      # wait for ask
    ask = ord(got)
    if ask == 0x79:
      # ACK
      return 1
    elif ask == 0x1F:
      # NACK
      raise CmdException("Chip replied with a NACK during %s" % info)

    # Unknown response
    raise CmdException("Unrecognised response 0x%x to %s" % (ask, info))

  def reset(self):
    self.sp.setDTR(1)
    time.sleep(0.5)
    self.sp.setDTR(0)
    time.sleep(0.5)

  def initChip(self):
    # Set boot
    self.sp.setRTS(0)
    self.reset()

    self.sp.write("\x7F")     # Syncro
    return self._wait_for_ack("Syncro")

  def releaseChip(self):
    self.sp.setRTS(1)
    self.reset()

  def cmdGeneric(self, cmd):
    self.sp.write(chr(cmd))
    self.sp.write(chr(cmd ^ 0xFF)) # Control byte
    return self._wait_for_ack(hex(cmd))

  def cmdGet(self):
    if self.cmdGeneric(0x00):
      mdebug(10, "*** Get command");
      len = ord(self.sp.read())
      version = ord(self.sp.read())
      mdebug(10, "  Bootloader version: "+hex(version))
      dat = map(lambda c: hex(ord(c)), self.sp.read(len))
      mdebug(10, "  Available commands: "+str(dat))
      self._wait_for_ack("0x00 end")
      return version
    else:
      raise CmdException("Get (0x00) failed")

  def cmdGetVersion(self):
    if self.cmdGeneric(0x01):
      mdebug(10, "*** GetVersion command")
      version = ord(self.sp.read())
      self.sp.read(2)
      self._wait_for_ack("0x01 end")
      mdebug(10, "  Bootloader version: "+hex(version))
      return version
    else:
      raise CmdException("GetVersion (0x01) failed")

  def cmdGetID(self):
    if self.cmdGeneric(0x02):
      mdebug(10, "*** GetID command")
      len = ord(self.sp.read())
      id = self.sp.read(len+1)
      self._wait_for_ack("0x02 end")
      return id
    else:
      raise CmdException("GetID (0x02) failed")


  def _encode_addr(self, addr):
    byte3 = (addr >> 0) & 0xFF
    byte2 = (addr >> 8) & 0xFF
    byte1 = (addr >> 16) & 0xFF
    byte0 = (addr >> 24) & 0xFF
    crc = byte0 ^ byte1 ^ byte2 ^ byte3
    return (chr(byte0) + chr(byte1) + chr(byte2) + chr(byte3) + chr(crc))


  def cmdReadMemory(self, addr, lng):
    assert(lng <= 256)
    if self.cmdGeneric(0x11):
      mdebug(10, "*** ReadMemory command")
      self.sp.write(self._encode_addr(addr))
      self._wait_for_ack("0x11 address failed")
      N = (lng - 1) & 0xFF
      crc = N ^ 0xFF
      self.sp.write(chr(N) + chr(crc))
      self._wait_for_ack("0x11 length failed")
      return map(lambda c: ord(c), self.sp.read(lng))
    else:
      raise CmdException("ReadMemory (0x11) failed")


  def cmdGo(self, addr):
    if self.cmdGeneric(0x21):
      mdebug(10, "*** Go command")
      self.sp.write(self._encode_addr(addr))
      self._wait_for_ack("0x21 go failed")
    else:
      raise CmdException("Go (0x21) failed")


  def cmdWriteMemory(self, addr, data):
    assert(len(data) <= 256)
    if self.cmdGeneric(0x31):
      mdebug(10, "*** Write memory command")
      self.sp.write(self._encode_addr(addr))
      self._wait_for_ack("0x31 address failed")
      #map(lambda c: hex(ord(c)), data)
      lng = (len(data)-1) & 0xFF
      mdebug(10, "  %s bytes to write" % [lng+1]);
      self.sp.write(chr(lng)) # len really
      crc = 0xFF
      for c in data:
        crc = crc ^ c
        self.sp.write(chr(c))
      self.sp.write(chr(crc))
      self._wait_for_ack("0x31 programming failed")
      mdebug(10, "  Write memory done")
    else:
      raise CmdException("Write memory (0x31) failed")


  def cmdEraseMemory(self, sectors = None):
    if self.cmdGeneric(0x43):
      mdebug(10, "*** Erase memory command")
      if sectors is None:
        # Global erase
        self.sp.write(chr(0xFF))
        self.sp.write(chr(0x00))
      else:
        # Sectors erase
        self.sp.write(chr((len(sectors)-1) & 0xFF))
        crc = 0xFF
        for c in sectors:
          crc = crc ^ c
          self.sp.write(chr(c))
        self.sp.write(chr(crc))
      self._wait_for_ack("0x43 erasing failed")
      mdebug(10, "  Erase memory done")
    else:
      raise CmdException("Erase memory (0x43) failed")

  GLOBAL_ERASE_TIMEOUT_SECONDS = 20   # This takes a while
  def cmdExtendedEraseMemory(self):
    if self.cmdGeneric(0x44):
      mdebug(10, "*** Extended erase memory command")
      # Global mass erase
      mdebug(5, "Global mass erase; this may take a while")
      self.sp.write(chr(0xFF))
      self.sp.write(chr(0xFF))
      # Checksum
      self.sp.write(chr(0x00))
      self._wait_for_ack("0x44 extended erase failed",
                         timeout=self.GLOBAL_ERASE_TIMEOUT_SECONDS)
      mdebug(10, "    Extended erase memory done")
    else:
      raise CmdException("Extended erase memory (0x44) failed")
          
  def cmdWriteProtect(self, sectors):
    if self.cmdGeneric(0x63):
      mdebug(10, "*** Write protect command")
      self.sp.write(chr((len(sectors)-1) & 0xFF))
      crc = 0xFF
      for c in sectors:
        crc = crc ^ c
        self.sp.write(chr(c))
      self.sp.write(chr(crc))
      self._wait_for_ack("0x63 write protect failed")
      mdebug(10, "  Write protect done")
    else:
      raise CmdException("Write Protect memory (0x63) failed")

  def cmdWriteUnprotect(self):
    if self.cmdGeneric(0x73):
      mdebug(10, "*** Write Unprotect command")
      self._wait_for_ack("0x73 write unprotect failed")
      self._wait_for_ack("0x73 write unprotect 2 failed")
      mdebug(10, "  Write Unprotect done")
    else:
      raise CmdException("Write Unprotect (0x73) failed")

  def cmdReadoutProtect(self):
    if self.cmdGeneric(0x82):
      mdebug(10, "*** Readout protect command")
      self._wait_for_ack("0x82 readout protect failed")
      self._wait_for_ack("0x82 readout protect 2 failed")
      mdebug(10, "  Read protect done")
    else:
      raise CmdException("Readout protect (0x82) failed")

  def cmdReadoutUnprotect(self):
    if self.cmdGeneric(0x92):
      mdebug(10, "*** Readout Unprotect command")
      self._wait_for_ack("0x92 readout unprotect failed")
      self._wait_for_ack("0x92 readout unprotect 2 failed")
      mdebug(10, "  Read Unprotect done")
    else:
      raise CmdException("Readout unprotect (0x92) failed")

# Complex commands section

  def readMemory(self, addr, lng):
    data = []
    if usepbar:
      widgets = ['Reading: ', Percentage(),', ', ETA(), ' ', Bar()]
      pbar = ProgressBar(widgets=widgets,maxval=lng, term_width=79).start()
    
    while lng > 256:
      if usepbar:
        pbar.update(pbar.maxval-lng)
      else:
        mdebug(5, "Read %(len)d bytes at 0x%(addr)X" % {'addr': addr, 'len': 256})
      data = data + self.cmdReadMemory(addr, 256)
      addr = addr + 256
      lng = lng - 256
    if usepbar:
      pbar.update(pbar.maxval-lng)
      pbar.finish()
    else:
      mdebug(5, "Read %(len)d bytes at 0x%(addr)X" % {'addr': addr, 'len': 256})
    data = data + self.cmdReadMemory(addr, lng)
    return data

  def writeMemory(self, addr, data):
    lng = len(data)
    if usepbar:
      widgets = ['Writing: ', Percentage(),' ', ETA(), ' ', Bar()]
      pbar = ProgressBar(widgets=widgets, maxval=lng, term_width=79).start()
    
    offs = 0
    while lng > 256:
      if usepbar:
        pbar.update(pbar.maxval-lng)
      else:
        mdebug(5, "Write %(len)d bytes at 0x%(addr)X" % {'addr': addr, 'len': 256})
      self.cmdWriteMemory(addr, data[offs:offs+256])
      offs = offs + 256
      addr = addr + 256
      lng = lng - 256
    if usepbar:
      pbar.update(pbar.maxval-lng)
      pbar.finish()
    else:
      mdebug(5, "Write %(len)d bytes at 0x%(addr)X" % {'addr': addr, 'len': 256})
    self.cmdWriteMemory(addr, data[offs:offs+lng] + ([0xFF] * (256-lng)) )

  def __init__(self):
    pass


def usage():
  print """Usage: %s [-hqVewvr] [-l length] [-p port] [-b baud] [-a addr] [file.bin]
  -h      This help
  -q      Quiet
  -V      Verbose
  -e      Erase
  -w      Write
  -v      Verify
  -r      Read
  -l length   Length of read
  -p port   Serial port (default: /dev/tty.usbserial-ftCYPMYJ)
  -b baud   Baud speed (default: 115200)
  -a addr   Target address

  ./stm32loader.py -e -w -v example/main.bin

  """ % sys.argv[0]


if __name__ == "__main__":
  
  # Import Psyco if available
  try:
    import psyco
    psyco.full()
    print "Using Psyco..."
  except ImportError:
    pass

  conf = {
      'port': '/dev/ttyUSB0',
      'baud': 115200,
      'address': 0x08000000,
      'erase': 0,
      'write': 0,
      'verify': 0,
      'read': 0,
      'len': 1000,
      'fname':'',
    }

# http://www.python.org/doc/2.5.2/lib/module-getopt.html

  try:
    opts, args = getopt.getopt(sys.argv[1:], "hqVewvrp:b:a:l:")
  except getopt.GetoptError, err:
    # print help information and exit:
    print str(err) # will print something like "option -a not recognized"
    usage()
    sys.exit(2)

  QUIET = 5

  for o, a in opts:
    if o == '-V':
      QUIET = 10
    elif o == '-q':
      QUIET = 0
    elif o == '-h':
      usage()
      sys.exit(0)
    elif o == '-e':
      conf['erase'] = 1
    elif o == '-w':
      conf['write'] = 1
    elif o == '-v':
      conf['verify'] = 1
    elif o == '-r':
      conf['read'] = 1
    elif o == '-p':
      conf['port'] = a
    elif o == '-b':
      conf['baud'] = eval(a)
    elif o == '-a':
      conf['address'] = eval(a)
    elif o == '-l':
      conf['len'] = eval(a)
#    elif o == '-f':
#      conf['fname'] = a
    else:
      assert False, "unhandled option"

  cmd = CommandInterface()
  cmd.open(conf['port'], conf['baud'])
  mdebug(10, "Open port %(port)s, baud %(baud)d" % {'port':conf['port'], 'baud':conf['baud']})
  try:
    try:
      cmd.initChip()
    except:
      print "Can't init. Ensure that BOOT0 is enabled and reset device"

    bootversion = cmd.cmdGet()
    mdebug(0, "Bootloader version %X" % bootversion)
    mdebug(0, "Chip id `%s'" % str(map(lambda c: hex(ord(c)), cmd.cmdGetID())))
#  cmd.cmdWriteProtect([0, 1])

    if (conf['write'] or conf['verify']):
      data = map(lambda c: ord(c), file(args[0], 'rb').read())

    if conf['erase']:
      if bootversion < 0x30:
        cmd.cmdEraseMemory()
      else:
        cmd.cmdExtendedEraseMemory()

    if conf['write']:
      cmd.writeMemory(conf['address'], data)
    if conf['verify']:
      verify = cmd.readMemory(conf['address'], len(data))
      if(data == verify):
        print "Verification OK"
      else:
        print "Verification FAILED"
        print str(len(data)) + ' vs ' + str(len(verify))
        for i in xrange(0, len(data)):
          if data[i] != verify[i]:
            print hex(i) + ': ' + hex(data[i]) + ' vs ' + hex(verify[i])

    if not conf['write'] and conf['read']:
      rdata = cmd.readMemory(conf['address'], conf['len'])
#      file(conf['fname'], 'wb').write(rdata)
      file(args[0], 'wb').write(''.join(map(chr,rdata)))

    cmd.cmdGo(conf['address'])
  finally:
    cmd.releaseChip()




#!/usr/bin/python2.5
#
# Copyright 2014 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
# See http://creativecommons.org/licenses/MIT/ for more information.
#
# -----------------------------------------------------------------------------
#
# Lookup table definitions.

import numpy

"""----------------------------------------------------------------------------
LFO and envelope increments.
----------------------------------------------------------------------------"""

lookup_tables = []
lookup_tables_32 = []

sample_rate = 31089
excursion = 1 << 32
num_values = 257

# Create lookup table for envelope times (x^0.25).
max_time = 12.0  # seconds
min_time = 0.0005
gamma = 0.25
min_increment = excursion / (max_time * sample_rate)
max_increment = excursion / (min_time * sample_rate)

rates = numpy.linspace(numpy.power(max_increment, -gamma),
                       numpy.power(min_increment, -gamma), num_values)

values = numpy.power(rates, -1/gamma).astype(int)
lookup_tables_32.append(
    ('env_increments', values)
)



"""----------------------------------------------------------------------------
Envelope curves
-----------------------------------------------------------------------------"""

env_linear = numpy.arange(0, 257.0) / 256.0
env_linear[-1] = env_linear[-2]
env_quartic = env_linear ** 3.32
env_expo = 1.0 - numpy.exp(-4 * env_linear)

lookup_tables.append(('env_linear', env_linear / env_linear.max() * 65535.0))
lookup_tables.append(('env_expo', env_expo / env_expo.max() * 65535.0))
lookup_tables.append(('env_quartic', env_quartic / env_quartic.max() * 65535.0))
lookup_tables.append(('square_root', (env_linear ** 0.5) * 65535.0))



"""----------------------------------------------------------------------------
SVF coefficients
----------------------------------------------------------------------------"""

cutoff = 440.0 * 2 ** ((numpy.arange(0, 257) - 69) / 12.0)
f = cutoff / sample_rate
f[f > 1 / 8.0] = 1 / 8.0
f = 2 * numpy.sin(numpy.pi * f)
resonance = numpy.arange(0, 257) / 257.0
damp = numpy.minimum(2 * (1 - resonance ** 0.25),
       numpy.minimum(2, 2 / f - f * 0.5))

lookup_tables.append(
    ('svf_cutoff', f * 32767.0)
)

lookup_tables.append(
    ('svf_damp', damp * 32767.0)
)



"""----------------------------------------------------------------------------
Vactrol attack/decay time
----------------------------------------------------------------------------"""

vactrol_time = 0.001 * 10 ** (numpy.arange(0, 128 * 5) / 128.0)
vactrol_time[0] = 0.0001
vactrol_time[1] = 0.0002
vactrol_time[2] = 0.0005
vactrol_time[3] = 0.001
filter_coefficients = 1.0 - numpy.exp(-1 / (vactrol_time * sample_rate))
lookup_tables_32.append(
    ('lp_coefficients', excursion / 2 * filter_coefficients)
)



"""----------------------------------------------------------------------------
2164 gain
----------------------------------------------------------------------------"""

gains = (numpy.arange(0, 257) / 256.0) * 3.3
gains =10 ** (-1.5 * gains)
lookup_tables.append(
    ('2164_gain', gains * 32767.0)
)



"""----------------------------------------------------------------------------
Compressor tables
----------------------------------------------------------------------------"""

t = (numpy.arange(0, 257) / 256.0)
lookup_tables_32 += [
    ('exp2', 65536.0 * 2 ** t),
    ('log2', 65536.0 * numpy.log2(256 + 256 * t))]

lookup_tables += [
    ('compressor_ratio', 256 / (24 * t * t + 1)),
    ('soft_knee', (t ** 3.0) * 65535.0)
]



"""----------------------------------------------------------------------------
Rate control
----------------------------------------------------------------------------"""

t = numpy.arange(0, 257) / 256.0
t /= (20 / 100.0 / 3.3)
f = 2.0 ** t
f /= f[-1]
f *= 0.02 * (1 << 24)

lookup_tables_32.append(('lorenz_rate', f))



# Copyright 2012 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
# See http://creativecommons.org/licenses/MIT/ for more information.
#
# -----------------------------------------------------------------------------
#
# Lookup table definitions.

import numpy

lookup_tables = []
lookup_tables_signed = []
lookup_tables_32 = []

sample_rate = 96000
excursion = 65536 * 65536.0

# Create table for pitch.
a4_midi = 69
a4_pitch = 440.0
highest_octave = 128
notes = numpy.arange(
    highest_octave * 128.0,
    (highest_octave + 12) * 128.0 + 16,
    16)
pitches = a4_pitch * 2 ** ((notes - a4_midi * 128) / (128 * 12))
increments = excursion / sample_rate * pitches
delays = sample_rate / pitches * 65536 * 4096

lookup_tables_32.append(
    ('oscillator_increments', increments.astype(int)))

lookup_tables_32.append(
    ('oscillator_delays', delays.astype(int)))



"""----------------------------------------------------------------------------
Resonator coefficients
----------------------------------------------------------------------------"""

cutoff = 440.0 * 2 ** ((numpy.arange(0, 129) - 69) / 12.0)
f = cutoff / (sample_rate / 2)
max_resonance = 0.99985
f[f > 0.25] = 0.25
bandpass_coeff_1 = -2 * numpy.cos(2 * numpy.pi * f)
bandpass_coeff_gain = []

for f in list(f):
  sample = 1.0
  y1 = 0.0
  y2 = 0.0
  n = numpy.arange(2000)
  response = numpy.sin((n + 1) * 2 * numpy.pi * f) / numpy.sin(2 * numpy.pi * f)
  response *= max_resonance ** n
  response /= (2 * f) ** 0.5
  gain = numpy.abs(response).max()
  bandpass_coeff_gain.append(gain)

bandpass_coeff_gain = numpy.maximum(1,
    numpy.minimum(
        256,
          16384.0 / numpy.array(bandpass_coeff_gain)))

lookup_tables.append(
    ('resonator_coefficient', -bandpass_coeff_1 * 32768.0)
)

lookup_tables.append(
    ('resonator_scale', bandpass_coeff_gain)
)


"""----------------------------------------------------------------------------
SVF coefficients
----------------------------------------------------------------------------"""

cutoff = 440.0 * 2 ** ((numpy.arange(0, 257) - 69) / 12.0)
f = cutoff / sample_rate
f[f > 1 / 8.0] = 1 / 8.0
f = 2 * numpy.sin(numpy.pi * f)
resonance = numpy.arange(0, 257) / 260.0
damp = numpy.minimum(2 * (1 - resonance ** 0.25),
       numpy.minimum(2, 2 / f - f * 0.5))

lookup_tables.append(
    ('svf_cutoff', f * 32767.0)
)

lookup_tables.append(
    ('svf_damp', damp * 32767.0)
)

lookup_tables.append(
    ('svf_scale', ((damp / 2) ** 0.5) * 32767.0)
)


"""----------------------------------------------------------------------------
Envelope for granular synthesis
----------------------------------------------------------------------------"""

granular_envelope = list(numpy.hanning(257) * 32767.0)
granular_envelope += [0] * 256
lookup_tables.append(
    ('granular_envelope', granular_envelope)
)

granular_envelope_rate = 2 ** (numpy.arange(0, 257) / 64.0) * (1 << 14)
lookup_tables.append(
    ('granular_envelope_rate', granular_envelope_rate / 8)
)


"""----------------------------------------------------------------------------
Bowing envelope and friction curve
----------------------------------------------------------------------------"""

attack = numpy.linspace(0, 1, int(sample_rate * 0.025 / 4)) * 0.2 * 32768
decay = numpy.linspace(1, 0.8, int(sample_rate * 0.005 / 4)) * 0.2 * 32768
bowing_envelope = list(attack) + list(decay)
# Add a guard to factor the border check out of the sample block loop
bowing_envelope += [decay[-1]] * 32
lookup_tables.append(
    ('bowing_envelope', bowing_envelope)
)

delta = numpy.arange(0, 257) / 64.0
friction = 1 / ((numpy.abs(delta) + 0.75) ** 4)
friction = numpy.minimum(friction, 1.0)
lookup_tables.append(
    ('bowing_friction', friction * 32768.0)
)

attack = numpy.linspace(0, 1, int(sample_rate * 0.005 / 4)) * 1.3 * 16384
decay = numpy.linspace(1, 0.8, int(sample_rate * 0.01 / 4)) * 1.3 * 16384
blowing_envelope = list(attack) + list(decay)
# Add a guard to factor the border check out of the sample block loop
blowing_envelope += [decay[-1]] * 32
lookup_tables.append(
    ('blowing_envelope', blowing_envelope)
)

delta = numpy.arange(0, 257) / 128.0
jet = delta ** 3 - delta
jet = numpy.minimum(jet, 1.0)
lookup_tables_signed.append(
    ('blowing_jet', jet * 32767.0)
)

flute_body_filter = 4096 * numpy.minimum(
    0.7, 0.4 * 2 ** ((numpy.arange(0, 128) - 69) / 12.0))
lookup_tables.append(
    ('flute_body_filter', flute_body_filter)
)


"""----------------------------------------------------------------------------
Quantizer for FM frequencies.
----------------------------------------------------------------------------"""

fm_frequency_ratios = [ 0.125, 0.25, 0.5, 0.5 * 2 ** (16 / 1200.0),
  numpy.sqrt(2) / 2, numpy.pi / 4, 1.0, 1.0 * 2 ** (16 / 1200.0), numpy.sqrt(2),
  numpy.pi / 2, 7.0 / 4, 2, 2 * 2 ** (16 / 1200.0), 9.0 / 4, 11.0 / 4,
  2 * numpy.sqrt(2), 3, numpy.pi, numpy.sqrt(3) * 2, 4, numpy.sqrt(2) * 3,
  numpy.pi * 3 / 2, 5, numpy.sqrt(2) * 4, 8]

scale = []
for ratio in fm_frequency_ratios:
  ratio = 256 * 12 * numpy.log2(ratio) + 16384
  scale.extend([ratio, ratio, ratio])

target_size = int(2 ** numpy.ceil(numpy.log2(len(scale))))
while len(scale) < target_size:
  gap = numpy.argmax(numpy.diff(scale))
  scale = scale[:gap + 1] + [(scale[gap] + scale[gap + 1]) / 2] + \
      scale[gap + 1:]

scale.append(scale[-1])

lookup_tables.append(
    ('fm_frequency_quantizer', scale)
)


"""----------------------------------------------------------------------------
Simulates VCO detuning
----------------------------------------------------------------------------"""

modified_pitch = []

for i in xrange(257):
  frequency = 440 * 2 ** ((i / 2.0 - 69) / 12.0)
  
  # Simulates an offset current in the integrator.
  frequency -= 0.6
  
  # Simulates the integrator cap reset time.
  time = 1 / frequency
  time += 9e-6
  frequency = 1 / time
  
  midi_pitch = 128 * (69 + 12 * numpy.log2(frequency / 440.0))
  midi_pitch = max(midi_pitch, 0)
  modified_pitch.append(midi_pitch)

modified_pitch = numpy.array(modified_pitch)
modified_pitch += (60 << 7) - modified_pitch[120]

lookup_tables.append(
    ('vco_detune', modified_pitch)
)


"""----------------------------------------------------------------------------
Bell envelopes for VOSIM and FOF
----------------------------------------------------------------------------"""

def bell(size, ratio):
  n = size / ratio
  first_half = numpy.hanning(n * 2)[:n] * 65535
  r = size - n
  second_half = numpy.hanning(r * 2)[r:] * 65535
  bell = list(first_half) + list(second_half) + [0]
  return bell

lookup_tables.append(('bell', bell(256, 16)))


"""----------------------------------------------------------------------------
Envelope increments.
----------------------------------------------------------------------------"""

sample_rate = 48000
control_rate = sample_rate / 24.0
max_time = 12.0  # seconds
min_time = 3.0 / control_rate  # seconds
gamma = 0.175
min_increment = excursion / (max_time * control_rate)
max_increment = excursion / (min_time * control_rate)
rates = numpy.linspace(numpy.power(max_increment, -gamma),
                       numpy.power(min_increment, -gamma), 128)
values = numpy.power(rates, -1/gamma).astype(int)
lookup_tables_32.append(
    ('env_portamento_increments', values)
)


"""----------------------------------------------------------------------------
Envelope curves
-----------------------------------------------------------------------------"""

env_linear = numpy.arange(0, 257.0) / 256.0
env_expo = 1.0 - numpy.exp(-4 * env_linear)

lookup_tables.append(('env_expo', env_expo / env_expo.max() * 65535.0))



#!/usr/bin/python2.5
#
# Copyright 2012 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -----------------------------------------------------------------------------
#
# Waveshaper lookup tables.

import numpy

waveshapers = []

def scale(x, min=-32766, max=32766, center=True):
  if center:
    x -= x.mean()
  mx = numpy.abs(x).max()
  x = (x + mx) / (2 * mx)
  x = x * (max - min) + min
  x = numpy.round(x)
  return x.astype(int)


x = ((numpy.arange(0, 257) / 128.0 - 1.0))
x[-1] = x[-2]
violent_overdrive = numpy.tanh(8.0 * x)
overdrive = numpy.tanh(5.0 * x)
moderate_overdrive = numpy.tanh(2.0 * x)

# Wavefolder curves from the first version
# tri_fold = numpy.abs(4.0 * x - numpy.round(4.0 * x)) * numpy.sign(x)
# sine_fold = numpy.sin(5 * numpy.pi * x)

tri_fold = numpy.sin(numpy.pi * (3 * x + (2 * x) ** 3))

# In v1.4 RC
window = numpy.exp(-x * x * 4) ** 1.5
sine = numpy.sin(8 * numpy.pi * x)
sine_fold = sine * window + numpy.arctan(3 * x) * (1 - window)
sine_fold /= numpy.abs(sine_fold).max()

# Another curve for the sine wavefolder inspired by the uFold response
# frequency = 4 / (1 + (1.5 * x) ** 2) ** 2
# window = numpy.exp(-24 * numpy.maximum(numpy.abs(x) - 0.25, 0) ** 2)
# sine = numpy.sin(2 * numpy.pi * frequency * x) * window
# cubic = (x ** 3 + 0.5 * x ** 2 + 0.5 * x) / 3
# knee = numpy.minimum(x + 0.7, 0.0)
# sine_fold = sine + cubic + knee

waveshapers.append(('moderate_overdrive', scale(moderate_overdrive)))
waveshapers.append(('violent_overdrive', scale(violent_overdrive)))
waveshapers.append(('sine_fold', scale(sine_fold, center=False)))
waveshapers.append(('tri_fold', scale(tri_fold)))



# Anti-aliasing filter design
# Copyright (C) 2020 Tyler Coy
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from scipy import signal
import math
import cog

# fs: base sampling rate in Hz
# os: oversampling factor
# fpass: passband corner in Hz
# fstop: stopband corner in Hz
# rpass: passband ripple in dB
# rstop: stopband attenuation in dB
def design(fs, os, fpass, fstop, rpass, rstop):
    wp = fpass / (fs * os)
    ws = min(0.5, fstop / (fs * os))

    n, wc = signal.ellipord(wp*2, ws*2, rpass, rstop)

    # We are using second-order sections, so if the filter order would have
    # been odd, we can bump it up by 1 for 'free'
    n = 2 * int(math.ceil(n / 2))

    # Non-oversampled sampling rates result in 0-order filters, since there
    # is no spectral content above fs/2. Bump these up to order 2 so we
    # get some rolloff.
    n = max(2, n)
    z, p, k = signal.ellip(n, rpass, rstop, wc, output='zpk')

    if n % 2 == 0:
        # DC gain is -rpass for even-order filters, so amplify by rpass
        k *= math.pow(10, rpass / 20)
    sos = signal.zpk2sos(z, p, k)

    cascade = FilterDescription((fs, os, n, wc/2, ws, sos))
    return cascade

class FilterDescription(tuple):
    def __init__(self, desc):
        (fs, os, n, wc, ws, sos) = desc
        self.sample_rate = fs
        self.oversampling = os
        self.order = n
        self.wpass = wc
        self.wstop = ws
        self.fpass = wc * fs * os
        self.fstop = ws * fs * os
        self.sections = sos

def print_filter_cases(filters):
    for f in filters:
        fs = f.sample_rate
        factor = f.oversampling
        num_sections = len(f.sections)
        name = 'kFilter{}x{}'.format(fs, factor)
        cost = fs * factor * num_sections

        cog.outl('case {}: // o = {}, fp = {}, fst = {}, cost = {}'
            .format(fs, factor, int(f.fpass), int(f.fstop), cost))
        cog.outl('{')

        cog.outl('    const SOSCoefficients {}[{}] ='
            .format(name, num_sections))
        cog.outl('    {')
        print_coeff = lambda c: '{:.8e},'.format(c).ljust(17)
        for s in f.sections:
            b = ''.join([print_coeff(c) for c in s[:3]])
            a = ''.join([print_coeff(c) for c in s[4:]])
            cog.outl('        { {' + b + '}, {' + a + '} },')
        cog.outl('    };')

        cog.outl('    AAFilter<T>::filter_.Init({}, {});'
            .format(num_sections, name))

        cog.outl('    break;')
        cog.outl('}')



#!/usr/bin/python2.5
#
# Copyright 2014 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
# See http://creativecommons.org/licenses/MIT/ for more information.
#
# -----------------------------------------------------------------------------
#
# Lookup table definitions.

import numpy
import pylab
import scipy.signal

SAMPLE_RATE = 32000
CRITICAL_FREQUENCY = 12000.0
IR_LENGTH = 8192

filters = []

ratios = [2]
lengths = [31, 45, 63, 91]

configurations = []
for oversampled in [False]:
  for ratio in ratios:
    for length in lengths:
      configurations += [(oversampled, ratio, length)]

generate_figures = False
for oversampled, ratio, length in configurations:
  transition = 0.15 * 0.25 ** (length / 100.0) if oversampled else 0.4
  ir = scipy.signal.remez(
      length,
      [0, transition / ratio, 0.499 / ratio, 0.5], [1, 0])
  ir /= sum(ir)
  gain = 20 * numpy.log10(numpy.abs(numpy.fft.rfft(ir, IR_LENGTH)))
  f = numpy.arange(IR_LENGTH / 2 + 1) / float(IR_LENGTH) * SAMPLE_RATE * ratio
  bin_index = CRITICAL_FREQUENCY / ratio / SAMPLE_RATE * IR_LENGTH
  critical_gain = gain[int(round(bin_index))]

  name = 'filter_%s_%d_%d' % ('2x' if oversampled else '1x', ratio, length)
  if generate_figures:
    pylab.figure(figsize=(8, 12))
    pylab.subplot(211)
    pylab.plot(f, gain)
    pylab.xlim([0.0, SAMPLE_RATE])
    pylab.ylim([-75.0, 3.0])
    pylab.xlabel('Frequency (Hz)')
    pylab.ylabel('Gain (dB)')
    caption = 'Resampling filter, N=%d, Ratio=%d. Gain at %d Hz = %.2fdB'
    pylab.title(caption % (length, ratio, CRITICAL_FREQUENCY, critical_gain))
  
    pylab.subplot(212)
    pylab.plot(f, gain)
    pylab.xlim([0.0, 20000.0])
    pylab.ylim([-3.0, 1.0])
    pylab.xlabel('Frequency (Hz)')
    pylab.ylabel('Gain (dB)')
  
    pylab.savefig(name + '.pdf')
    pylab.close()
  filters += [(name, ir)]



#!/usr/bin/python2.5
#
# Copyright 2014 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
# See http://creativecommons.org/licenses/MIT/ for more information.
#
# -----------------------------------------------------------------------------
#
# Lookup table definitions.

import numpy

lookup_tables = []
int16_lookup_tables = []
uint32_lookup_tables = []

SAMPLE_RATE = 32000.0


"""----------------------------------------------------------------------------
Sine table
----------------------------------------------------------------------------"""

WAVETABLE_SIZE = 4096
t = numpy.arange(0.0, WAVETABLE_SIZE + 1) / WAVETABLE_SIZE
t[-1] = t[0]

x = numpy.sin(2 * numpy.pi * t)
lookup_tables += [('sine', x)]



"""----------------------------------------------------------------------------
Coefficients for approximate filter, 32Hz to 16kHz ; Q = 0.5 to 500
----------------------------------------------------------------------------"""

frequency = 32 * (10 ** (2.7 * numpy.arange(0, 257) / 256))
frequency /= SAMPLE_RATE

frequency[frequency >= 0.499] = 0.499

g = numpy.tan(numpy.pi * frequency)
r = 2.0
h = 1.0 / (1.0 + r * g + g * g)
gain = (0.42 / frequency) * (4 ** (frequency * frequency))
r = 1 / (0.5 * 10 ** (3.0 * numpy.arange(0, 257) / 256))

lookup_tables += [
  ('approx_svf_gain', gain),
  ('approx_svf_g', g),
  ('approx_svf_r', r),
  ('approx_svf_h', h)]



"""----------------------------------------------------------------------------
Exponentials covering several decades in 256 steps, with safeguard
----------------------------------------------------------------------------"""

x = numpy.arange(0, 257) / 256.0
lookup_tables += [('4_decades', 10 ** (4 * x))]



"""----------------------------------------------------------------------------
3dB/V table for accent/strength control
----------------------------------------------------------------------------"""

x = numpy.arange(0, 257) / 256.0
coarse = 10 ** (1.5 * (x - 0.5))
fine = 10 ** (x * numpy.log10(coarse[1] / coarse[0]))
lookup_tables += [('accent_gain_coarse', coarse)]
lookup_tables += [('accent_gain_fine', fine)]



"""----------------------------------------------------------------------------
dB brightness table
----------------------------------------------------------------------------"""

x = numpy.arange(0, 513) / 512.0
x[0] = x[1]
x[-1] = x[-2]
brightness = (9 + numpy.log2(x)) / 9
int16_lookup_tables += [('db_led_brightness', brightness * 256.0)]



"""----------------------------------------------------------------------------
Stiffness table.
----------------------------------------------------------------------------"""

geometry = numpy.arange(0, 257) / 256.0
stiffness = geometry + 0
for i, g in enumerate(geometry):
  if g < 0.25:
    g = 0.25 - g
    stiffness[i] = -g * 0.25
  elif g < 0.3:
    stiffness[i] = 0.0
  elif g < 0.9:
    g -= 0.3
    g /= 0.6
    stiffness[i] = 0.01 * 10 ** (g * 2.005) - 0.01
  else:
    g -= 0.9
    g /= 0.1
    g *= g
    stiffness[i] = 1.5 - numpy.cos(g * numpy.pi) / 2.0

stiffness[-1] = 2.0
stiffness[-2] = 2.0
lookup_tables += [('stiffness', stiffness)]



"""----------------------------------------------------------------------------
Envelope increments and curves.
----------------------------------------------------------------------------"""

TABLE_SIZE = 256
t = numpy.arange(0.0, TABLE_SIZE + 2) / TABLE_SIZE
t[-1] = t[-2] = 1.0

control_rate = SAMPLE_RATE / 16.0
max_time = 8.0  # seconds
min_time = 0.0005
gamma = 0.175
min_increment = 1.0 / (max_time * control_rate)
max_increment = 1.0 / (min_time * control_rate)
a = numpy.power(max_increment, -gamma)
b = numpy.power(min_increment, -gamma)

lookup_tables.append(
    ('env_increments', numpy.power(a + (b - a) * t, -1 / gamma))
)

env_linear = t
env_quartic = t ** 3.32
env_expo = 1.0 - numpy.exp(-4 * t)

lookup_tables.append(('env_linear', env_linear / env_linear.max()))
lookup_tables.append(('env_expo', env_expo / env_expo.max()))
lookup_tables.append(('env_quartic', env_quartic / env_quartic.max()))



"""----------------------------------------------------------------------------
MIDI to normalized frequency table.
----------------------------------------------------------------------------"""

TABLE_SIZE = 256

midi_note = numpy.arange(0, TABLE_SIZE) - 48
frequency = 440 * 2 ** ((midi_note - 69) / 12.0)
max_frequency = min(12000, SAMPLE_RATE / 2)
frequency[frequency >= max_frequency] = max_frequency
frequency /= SAMPLE_RATE

semitone = 2 ** (numpy.arange(0, TABLE_SIZE) / 256.0 / 12.0)


lookup_tables.append(('midi_to_f_high', frequency))
lookup_tables.append(('midi_to_increment_high', frequency * (1 << 32)))
lookup_tables.append(('midi_to_f_low', semitone))



"""----------------------------------------------------------------------------
Quantizer for FM frequencies.
----------------------------------------------------------------------------"""

fm_frequency_ratios = [ 0.5, 0.5 * 2 ** (16 / 1200.0),
  numpy.sqrt(2) / 2, numpy.pi / 4, 1.0, 1.0 * 2 ** (16 / 1200.0), numpy.sqrt(2),
  numpy.pi / 2, 7.0 / 4, 2, 2 * 2 ** (16 / 1200.0), 9.0 / 4, 11.0 / 4,
  2 * numpy.sqrt(2), 3, numpy.pi, numpy.sqrt(3) * 2, 4, numpy.sqrt(2) * 3,
  numpy.pi * 3 / 2, 5, numpy.sqrt(2) * 4, 8]

scale = []
for ratio in fm_frequency_ratios:
  ratio = 12 * numpy.log2(ratio)
  scale.extend([ratio, ratio, ratio])

target_size = int(2 ** numpy.ceil(numpy.log2(len(scale))))
while len(scale) < target_size:
  gap = numpy.argmax(numpy.diff(scale))
  scale = scale[:gap + 1] + [(scale[gap] + scale[gap + 1]) / 2] + \
      scale[gap + 1:]

scale.append(scale[-1])

lookup_tables.append(
    ('fm_frequency_quantizer', scale)
)



"""----------------------------------------------------------------------------
Quantizer for FM frequencies.
----------------------------------------------------------------------------"""

detune_ratios = [-24, -12, -11.95, -5.0, -0.05, 0.0, 0.05, 7.0, 12.0, 19.0, 24.0]

scale = []
for ratio in detune_ratios:
  scale.extend([ratio, ratio, ratio])

target_size = int(2 ** numpy.ceil(numpy.log2(len(scale))))
while len(scale) < target_size:
  gap = numpy.argmax(numpy.diff(scale))
  scale = scale[:gap + 1] + [(scale[gap] + scale[gap + 1]) / 2] + \
      scale[gap + 1:]

scale.append(scale[-1])

lookup_tables.append(
    ('detune_quantizer', scale)
)



"""----------------------------------------------------------------------------
Delay compensation factor for SVF
----------------------------------------------------------------------------"""

ratio = 2.0 ** (numpy.arange(0, 257) / 12.0)
svf_shift = 2.0 * numpy.arctan(1.0 / ratio) / (2.0 * numpy.pi)
lookup_tables += [('svf_shift', svf_shift)]


#!/usr/bin/python2.5
#
# Copyright 2014 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
# See http://creativecommons.org/licenses/MIT/ for more information.
#
# -----------------------------------------------------------------------------
#
# Lookup table definitions.

import numpy
import pylab


def scale(x):
  xc = x - x.mean()
  abs_max = numpy.abs(xc).max()
  xc /= abs_max
  return xc



def fshift(x, shift):
  s = x[:-1] + 0
  s = numpy.fft.rfft(s)
  s[1:] *= shift
  xh = numpy.fft.irfft(s)
  xh -= xh.mean()
  return numpy.array(list(xh) + [xh[0]])



def make_quadrature(name, x, angle_1=0, angle_2=0.5):
  xc = fshift(x, numpy.exp(1j * angle_1 * numpy.pi))
  xh = fshift(x, numpy.exp(1j * angle_2 * numpy.pi))
  scale = max(numpy.abs(xc).max(), numpy.abs(xh).max())
  return [(name + '_i', xc / scale), (name + '_q', xh / scale)]



SAMPLE_RATE = 96000
WAVETABLE_SIZE = 1024

t = numpy.arange(WAVETABLE_SIZE + 1) / float(WAVETABLE_SIZE) * 2 * numpy.pi

sine = -numpy.sin(t)
harmonics = -numpy.sin(t) - 0.5 * numpy.sin(2 * t) + 0.5 * numpy.sin(5 * t)
buzzy = 0
for i in xrange(7):
  buzzy += numpy.sin((1 + i) * t + 1.012 * i) * numpy.sin(1.123 * i)

iq_waveforms = []
iq_waveforms += make_quadrature('sine', sine)
iq_waveforms += make_quadrature('harmonics', harmonics)
iq_waveforms += make_quadrature('buzzy', buzzy)



#!/usr/bin/python2.5
#
# Copyright 2013 Olivier Gillet.
#
# Author: Olivier Gillet (pichenettes@mutable-instruments.net)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -----------------------------------------------------------------------------
#
# Fsk encoder for converting firmware .bin files into .

import numpy
import optparse
import zlib

from avr_audio_bootloader import audio_stream_writer


class FskEncoder(object):
  
  def __init__(
      self,
      sample_rate=48000,
      pause_period=32,
      one_period=8,
      zero_period=4,
      packet_size=256):
    self._sr = sample_rate
    self._pause_period = pause_period
    self._one_period = one_period
    self._zero_period = zero_period
    self._packet_size = packet_size
    self._state = 1
    
  def _encode(self, symbol_stream):
    symbol_stream = numpy.array(symbol_stream)
    counts = [numpy.sum(symbol_stream == symbol) for symbol in range(3)]
    durations = [self._zero_period, self._one_period, self._pause_period]
    total_length = numpy.dot(durations, counts)
    signal = numpy.zeros((total_length, 1))
    state = self._state
    index = 0
    for symbol in symbol_stream:
      d = durations[symbol]
      signal[index:index + d] = state
      state = -state
      index += d
    assert index == signal.shape[0]
    self._state = state
    return signal
    
  def _code_blank(self, duration):
    num_symbols = int(duration * self._sr / self._pause_period) + 1
    return self._encode([2] * num_symbols)
    
  def _code_packet(self, data):
    assert len(data) <= self._packet_size
    if len(data) != self._packet_size:
      data = data + '\x00' * (self._packet_size - len(data))

    crc = zlib.crc32(data) & 0xffffffff

    data = map(ord, data)
    crc_bytes = [crc >> 24, (crc >> 16) & 0xff, (crc >> 8) & 0xff, crc & 0xff]
    bytes = [0x55] * 4 + data + crc_bytes
    
    symbol_stream = []
    for byte in bytes:
      mask = 0x80
      for _ in range(0, 8):
        symbol_stream.append(1 if (byte & mask) else 0)
        mask >>= 1
    
    return self._encode(symbol_stream)

  def code(self, data, page_size=1024, blank_duration=0.06):
    yield numpy.zeros((1.0 * self._sr, 1)).ravel()
    yield self._code_blank(1.0)
    if len(data) % page_size != 0:
      tail = page_size - (len(data) % page_size)
      data += '\xff' * tail
    
    offset = 0
    remaining_bytes = len(data)
    num_packets_written = 0
    while remaining_bytes:
      size = min(remaining_bytes, self._packet_size)
      yield self._code_packet(data[offset:offset+size])
      num_packets_written += 1
      if num_packets_written == page_size / self._packet_size:
        yield self._code_blank(blank_duration)
        num_packets_written = 0
      remaining_bytes -= size
      offset += size
    yield self._code_blank(1.0)


def main():
  parser = optparse.OptionParser()
  parser.add_option(
      '-s',
      '--sample_rate',
      dest='sample_rate',
      type='int',
      default=40000,
      help='Sample rate in Hz')
  parser.add_option(
      '-b',
      '--pause_period',
      dest='pause_period',
      type='int',
      default=64,
      help='Period (in samples) of a blank symbol')
  parser.add_option(
      '-n',
      '--one_period',
      dest='one_period',
      type='int',
      default=16,
      help='Period (in samples) of a one symbol')
  parser.add_option(
      '-z',
      '--zero_period',
      dest='zero_period',
      type='int',
      default=8,
      help='Period (in samples) of a zero symbol')
  parser.add_option(
      '-p',
      '--packet_size',
      dest='packet_size',
      type='int',
      default=256,
      help='Packet size in bytes')
  parser.add_option(
      '-g',
      '--page_size',
      dest='page_size',
      type='int',
      default=64,
      help='Flash page size')
  parser.add_option(
      '-k',
      '--blank_duration',
      dest='blank_duration',
      type='int',
      default=8,
      help='Duration of the blank between pages, in ms')
  parser.add_option(
      '-o',
      '--output_file',
      dest='output_file',
      default=None,
      help='Write output file to FILE',
      metavar='FILE')
  
  options, args = parser.parse_args()
  data = file(args[0], 'rb').read()
  if len(args) != 1:
    logging.fatal('Specify one, and only one firmware .bin file!')
    sys.exit(1)

  output_file = options.output_file
  if not output_file:
    if '.bin' in args[0]:
      output_file = args[0].replace('.bin', '.wav')
    else:
      output_file = args[0] + '.wav'

  encoder = FskEncoder(
      options.sample_rate,
      options.pause_period,
      options.one_period,
      options.zero_period,
      options.packet_size)
  writer = audio_stream_writer.AudioStreamWriter(
      output_file,
      options.sample_rate,
      16,
      1)

  blank_duration = options.blank_duration * 0.001
  for block in encoder.code(data, options.page_size, blank_duration):
    if len(block):
      writer.append(block)


if __name__ == '__main__':
  main()



#!/usr/bin/python2.5
#
# Copyright 2009 Olivier Gillet.
#
# Author: Olivier Gillet (ol.gillet@gmail.com)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -----------------------------------------------------------------------------
#
# Python module for loading/writing Hex files.

"""Intel .hex file loader/writer"""

import logging
import sys


def LoadHexFile(lines):
  """Loads a Hex file."""

  data = []
  for line_number, line in enumerate(lines):
    line = line.strip()
    if len(line) < 9:
      logging.info('Line %(line_number)d: line too short' % locals())
      return None

    if not all(x in '0123456789abcdefABCDEF' for x in line[1:]):
      logging.info('Line %(line_number)d: unknown character' % locals())
      return None

    bytes = [int(line[i:i+2], 16) for i in xrange(1, len(line), 2)]
    if bytes[0] != len(bytes) - 5:
      logging.info('Line %(line_number)d: invalid byte count' % locals())
      return None

    if sum(bytes) % 256 != 0:
      logging.info('Line %(line_number)d: corrupted line' % locals())
      return None

    if bytes[3] == 1:
      if bytes[0] != 0 or bytes[1] != 0 or bytes[2] != 0:
        logging.info('Line %(line_number)d: invalid end of file' % locals())
        return None
      else:
        break
    elif bytes[3] == 0:
      address = bytes[1] << 8 | bytes[2]
      padding_size = address + bytes[0] - len(data)
      if padding_size > 0:
        data += ['\x00'] * padding_size
      data[address:address + bytes[0]] = bytes[4:-1]
  return data


def WriteHexFile(data, file_object, chunk_size=32):
  """Writes a Hex file."""

  for address in xrange(0, len(data), chunk_size):
    chunk = data[address:address+chunk_size]
    chunk_len = len(chunk)
    address_l = address & 255
    address_h = address >> 8
    file_object.write(':%(chunk_len)02x%(address_h)02x%(address_l)02x00' % vars())
    file_object.write(''.join('%02x' % value for value in chunk))
    checksum = (-(chunk_len + address_l + address_h + sum(chunk))) & 255
    file_object.write('%02x\n' % checksum)
  file_object.write(':00000001FF\n')



#!/usr/bin/python2.5
#
# Copyright 2014 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
# See http://creativecommons.org/licenses/MIT/ for more information.
#
# -----------------------------------------------------------------------------
#
# Lookup table definitions.

import numpy
import pylab
import scipy.signal


def pole_pair_to_f_fq(pole_pair):
  fq = 1 - pole_pair.prod()
  f = -(2 - fq - (pole_pair.sum())) ** 0.5
  return f.real, fq.real



def modified_chamberlin(f, fq, x, mode='bp'):
  lp = 0.0
  bp = 0.0
  y = numpy.zeros(x.shape)
  x_ = 0.0
  coefficient = 1.0 if mode == 'bp' else 0.0
  for i in xrange(len(y)):
    lp += f * bp
    bp += -fq * bp -f * lp + (x[i] + x_ * coefficient)
    x_ = x[i]
    if mode =='bp':
      y[i] = fq * bp
    elif mode == 'lp':
      y[i] = f * lp
    elif mode == 'hp':
      y[i] = x_ - lp * f - bp * fq
  return y



SAMPLE_RATE = 96000
IR_SIZE = 2048


sample_rates = [SAMPLE_RATE / 12] * 13
sample_rates += [SAMPLE_RATE / 3] * 6
sample_rates += [SAMPLE_RATE] * 1
num_bands = len(sample_rates)

interval = 2 ** (1 / 3.0)
first_frequency = 110 / interval
frequencies = first_frequency * (interval ** numpy.arange(0, num_bands))

filters = []

responses = {}
reconstruction = {}

generate_figures = __name__ == '__main__'

for index, (frequency, sr) in enumerate(zip(frequencies, sample_rates)):
  if not sr in reconstruction:
    reconstruction[sr] = [0.0, 0.0]
    responses[sr] = []
  
  frequency = frequency / (sr * 0.5)
  
  if index == 0:
    w = frequency
    z, p, k = scipy.signal.cheby1(4, 0.5, w, 'lowpass', output='zpk')
    svf_mode = 'lp'
    gain = 1.0
  elif index == num_bands - 1:
    w = frequency
    z, p, k = scipy.signal.cheby1(4, 0.25, w, 'highpass', output='zpk')
    svf_mode = 'hp'
    gain = 21 * frequency
  else:
    w = [frequency / (interval ** 0.5), frequency * (interval ** 0.5)]
    z, p, k = scipy.signal.butter(2, w, 'bandpass', output='zpk')
    svf_mode = 'bp'
    gain = 0.25
  
  # Filter using direct form
  out = numpy.eye(IR_SIZE, 1).ravel()
  b, a = scipy.signal.zpk2tf(z, p, k)
  out = scipy.signal.lfilter(b, a, out)
  out = scipy.signal.lfilter(b, a, out)
  reconstruction[sr][0] += out
  responses[sr] += [out]
  
  # Filter using modified Chamberlin filter
  out = numpy.eye(IR_SIZE, 1).ravel() * gain
  
  coefficients = [0, 0, 0]
  for i in xrange(2):
    f, fq = pole_pair_to_f_fq(p[i*2:i*2 + 2])
    out = modified_chamberlin(f, fq, out, svf_mode)
    out = modified_chamberlin(f, fq, out, svf_mode)
    coefficients += [f, fq]
  
  delay = (numpy.arange(len(out)) * out * out).sum() / (out * out).sum()
  
  # Completely empirical fixes to the delay to maximize the flatness of the
  # total impulse response.
  if index == num_bands - 1:
    delay += 4
  
  coefficients[0] = SAMPLE_RATE / sr
  coefficients[1] = numpy.floor(delay)
  coefficients[2] = gain
  
  filters += [('%3.0f_%d' % (frequency * 0.5 * sr, sr), coefficients)]
  
  reconstruction[sr][1] += out



if generate_figures:
  pylab.figure(figsize=(20,8))
  n = len(responses.keys())
  for row, sr in enumerate(sorted(responses.keys())):
    f = numpy.arange(IR_SIZE / 2 + 1) / float(IR_SIZE) * sr
    for column, plots in enumerate([reconstruction[sr], responses[sr]]):
      pylab.subplot(2, n, column * n + row + 1)
      for r in plots:
        sy = numpy.log10(numpy.abs(numpy.fft.rfft(r)) + 1e-20) * 20.0
        pylab.semilogx(f, sy)
      pylab.xlim(80, sr / 2)
      pylab.ylim(-36, 12)
      pylab.xlabel('Frequency (Hz)')
      pylab.ylabel('Gain (dB)')
      if len(plots) == 2:
        pylab.ylim(-4, 3)
        #pylab.legend(['Direct form', 'Chamberlin'])
  pylab.savefig('filter_bank.pdf')
  # pylab.show()
  pylab.close()
  


#!/usr/bin/python2.5
#
# Copyright 2009 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -----------------------------------------------------------------------------
#
# Self-organizing map.

import numpy
import random


class SOM(object):
  
  def __init__(self, grid_size, radius, learning_rate):
    self._grid_size = grid_size
    self._grid_x = numpy.arange(0, grid_size * grid_size) % grid_size
    self._grid_y = numpy.arange(0, grid_size * grid_size) / grid_size
    self._radius = radius
    self._learning_rate = learning_rate
    self._codewords = None

  @staticmethod
  def standardize(x, std_power=1, axis=0, regularization=0.0):
    t, n = x.shape
    mean = x.mean(axis=axis)
    std = x.std(axis=axis) ** std_power + regularization
    return (x - mean) / std
    
  def classify(self, data):
    distances = ((self._codewords - data) ** 2).sum(axis=1)
    return distances.argmin(), distances
    
  def train(self, data, iterations=10000, seed=42):
    n, d = data.shape
    nodes = self._grid_size
    numpy.random.seed(seed)
    random.seed(seed)
    self._codewords = numpy.random.randn(nodes * nodes, d)
    self._error_history = []
    milestone = 2
    for i in xrange(iterations):
      if i == milestone:
        print 'iteration', i, 'of', iterations, '\t', \
            round(1000.0 * i / iterations) * 0.1, '%'
        milestone <<= 1
      radius = self._radius * 2 ** (-2 * float(i) / iterations)
      learning_rate = self._learning_rate * 2 ** (-7 * float(i) / iterations)
      
      # Pick a random vector.
      x = random.choice(data)
      
      # Find best matching unit.
      bmu, distances = self.classify(x)
      self._error_history.append(distances[bmu])
      
      # Compute neighborhood update function.
      delta_x = self._grid_x[bmu] - self._grid_x
      delta_y = self._grid_y[bmu] - self._grid_y
      rbf = numpy.exp(-(delta_x ** 2 + delta_y ** 2) / (radius * radius))
      rbf = numpy.tile(rbf.reshape((nodes * nodes, 1)), (1, d))
      update = rbf * (numpy.tile(x, (nodes * nodes, 1)) - self._codewords)
      self._codewords += learning_rate * update
    return self._error_history
    
  def checkpoint(self):
    numpy.save('weights', self._codewords)

  def resume(self):
    self._codewords = numpy.load('weights.npy')

  def plot(self, x):
    import pylab

    for i in xrange(x.shape[0]):
      _, d = self.classify(x[i, :])
      d = numpy.exp(-d)
      pylab.figure()
      pylab.imshow(
          d.reshape((self._grid_size, self._grid_size)),
          interpolation='nearest')
      pylab.savefig('response_%d.pdf' % i)



#!/usr/bin/env python
#
# Copyright 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import webapp2

from google.appengine.ext import webapp
from google.appengine.ext.webapp import util
from google.appengine.ext.webapp import template

from music.scala import scala

TEMPLATE_PATH = os.path.join(os.path.dirname(__file__), 'templates')



class SyxHandler(webapp.RequestHandler):
  def get(self):
    header = '\xf0\x7f\x7f\x08\x09\x03\x7f\x7f'
    data = self.request.get('data')
    body = ''.join([chr(int(data[x:x+2], 16)) for x in range(0, len(data), 2)])
    footer = '\xf7'
    self.response.headers['Content-Type'] = 'application/octet-stream'
    self.response.headers['Content-disposition'] = \
        'attachment; filename=scale.syx'
    self.response.out.write(header + body + footer)



class MainHandler(webapp.RequestHandler):
  def post(self):
    return self.get()
  
  def get(self):
    if self.request.get('scl'):
      source = self.request.get('scl').decode('iso-8859-1').encode('utf8')
      hide_if_error = True
    else:
      source = self.request.get('source')
      hide_if_error = False
    
    semitones = []
    try:
      error_message = ''
      notes = scala.parse(source)
      mapping = scala.assign_to_octave(notes)
      names = ['C', 'C#', 'D', 'D#', 'E', 'F', 'F#', 'G', 'G#', 'A', 'A#', 'B']
      mappings = []
      for i, (shift, note) in enumerate(zip(mapping, names)):
        mappings.append(
          '%s = %.2f cents' % (note, shift + i * 100.0))
      mapping_14bit = [int(round(8192 + 8192 * x / 100.0)) for x in mapping]
      mapping_hex = ['%02x%02x' % (m >> 7, m & 0x7f) for m in mapping_14bit]
      link = ''.join(mapping_hex)
      
    except scala.ParseError as p:
      error_message = p.message
      notes = []
      mappings = []
      link = ''
    
    if not source:
      error_message = ''
    
    self.response.headers['Content-Type'] = 'text/html'
    template_path = os.path.join(TEMPLATE_PATH, 'index.html')
    template_data = {}
    template_data['error_message'] = error_message
    template_data['notes'] = notes
    template_data['mappings'] = mappings
    template_data['link'] = link
    template_data['source'] = '' if hide_if_error and error_message else source
    self.response.out.write(template.render(template_path, template_data))

application = webapp.WSGIApplication([('/', MainHandler), ('/syx', SyxHandler)])



#!/usr/bin/python2.5
#
# Copyright 2015 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
# See http://creativecommons.org/licenses/MIT/ for more information.
#
# -----------------------------------------------------------------------------
#
# Lookup table definitions.

import numpy

lookup_tables = []
int16_lookup_tables = []
uint32_lookup_tables = []

SAMPLE_RATE = 48000.0



"""----------------------------------------------------------------------------
Sine table
----------------------------------------------------------------------------"""

WAVETABLE_SIZE = 4096
t = numpy.arange(0.0, WAVETABLE_SIZE + WAVETABLE_SIZE / 4 + 1) / WAVETABLE_SIZE
x = numpy.sin(2 * numpy.pi * t)
lookup_tables += [('sine', x)]



"""----------------------------------------------------------------------------
Exponentials covering several decades in 256 steps, with safeguard
----------------------------------------------------------------------------"""

x = numpy.arange(0, 257) / 256.0
lookup_tables += [('4_decades', 10 ** (4 * x))]



"""----------------------------------------------------------------------------
Delay compensation factor for SVF
----------------------------------------------------------------------------"""

ratio = 2.0 ** (numpy.arange(0, 257) / 12.0)
svf_shift = 2.0 * numpy.arctan(1.0 / ratio) / (2.0 * numpy.pi)
lookup_tables += [('svf_shift', svf_shift)]



"""----------------------------------------------------------------------------
Stiffness table.
----------------------------------------------------------------------------"""

structure = numpy.arange(0, 257) / 256.0
stiffness = structure + 0
for i, g in enumerate(structure):
  if g < 0.25:
    g = 0.25 - g
    stiffness[i] = -g * 0.25
  elif g < 0.3:
    stiffness[i] = 0.0
  elif g < 0.9:
    g -= 0.3
    g /= 0.6
    stiffness[i] = 0.01 * 10 ** (g * 2.005) - 0.01
  else:
    g -= 0.9
    g /= 0.1
    g *= g
    stiffness[i] = 1.5 - numpy.cos(g * numpy.pi) / 2.0

stiffness[-1] = 2.0
stiffness[-2] = 2.0
lookup_tables += [('stiffness', stiffness)]



"""----------------------------------------------------------------------------
Quantizer for FM frequencies.
----------------------------------------------------------------------------"""

fm_frequency_ratios = [ 0.5, 0.5 * 2 ** (16 / 1200.0),
  numpy.sqrt(2) / 2, numpy.pi / 4, 1.0, 1.0 * 2 ** (16 / 1200.0), numpy.sqrt(2),
  numpy.pi / 2, 7.0 / 4, 2, 2 * 2 ** (16 / 1200.0), 9.0 / 4, 11.0 / 4,
  2 * numpy.sqrt(2), 3, numpy.pi, numpy.sqrt(3) * 2, 4, numpy.sqrt(2) * 3,
  numpy.pi * 3 / 2, 5, numpy.sqrt(2) * 4, 8]

scale = []
for ratio in fm_frequency_ratios:
  ratio = 12 * numpy.log2(ratio)
  scale.extend([ratio, ratio, ratio])

target_size = int(2 ** numpy.ceil(numpy.log2(len(scale))))
while len(scale) < target_size:
  gap = numpy.argmax(numpy.diff(scale))
  scale = scale[:gap + 1] + [(scale[gap] + scale[gap + 1]) / 2] + \
      scale[gap + 1:]

scale.append(scale[-1])

lookup_tables.append(
    ('fm_frequency_quantizer', scale)
)



#!/usr/bin/python2.5
#
# Copyright 2014 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
# See http://creativecommons.org/licenses/MIT/ for more information.
#
# -----------------------------------------------------------------------------
#
# Waveform definitions.

import numpy

WAVETABLE_SIZE = 1024

waveforms = []

"""----------------------------------------------------------------------------
Sine wave
----------------------------------------------------------------------------"""

x = numpy.arange(0, WAVETABLE_SIZE + 1) / float(WAVETABLE_SIZE)
x[-1] = x[0]
sine = numpy.sin(2 * numpy.pi * x)
waveforms.append(('sine', (32767 * sine).astype(int)))


"""----------------------------------------------------------------------------
Waveshaper
----------------------------------------------------------------------------"""

x = numpy.arange(0, WAVETABLE_SIZE + 1) / (WAVETABLE_SIZE / 2.0) - 1.0
x[-1] = x[-2]

window = numpy.exp(-x * x * 4) ** 1.5
sine = numpy.sin(8 * numpy.pi * x)
sine_fold = sine * window + numpy.arctan(3 * x) * (1 - window)
sine_fold /= numpy.abs(sine_fold).max()

power = x ** 7
overdrive = numpy.tanh(5.0 * x)
overdrive /= numpy.abs(overdrive).max()
waveforms.append(('fold_power', numpy.round(32767 * power).astype(int)))
waveforms.append(('fold_sine', numpy.round(32767 * sine_fold).astype(int)))
waveforms.append(('overdrive', numpy.round(32767 * overdrive).astype(int)))


"""----------------------------------------------------------------------------
Surprise!
----------------------------------------------------------------------------"""

digits = file('peaks/data/digits.bin', 'rb').read()
digits = map(ord, digits)
waveforms_8 = [('digits', digits)]



#!/usr/bin/python2.5
#
# Copyright 2014 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
# See http://creativecommons.org/licenses/MIT/ for more information.
#
# -----------------------------------------------------------------------------
#
# Custom LFO waveshapes.

import numpy


def scale(x, min=0, max=255, center=True):
  mx = x.max()
  mn = x.min()
  x = (x - mn) / (mx - mn)
  x = numpy.round(x * (max - min) + min)
  target_type = numpy.uint8
  x[x < numpy.iinfo(target_type).min] = numpy.iinfo(target_type).min
  x[x > numpy.iinfo(target_type).max] = numpy.iinfo(target_type).max
  return x.astype(target_type)


custom_lfos = []

t = numpy.arange(0, 256) / 256.0
ramp = t
triangle = 2 * t * (t < 0.5) + (2.0 - 2 * t) * (t >= 0.5)
square = (t < 0.5)
bipolar_triangle = 2 * triangle - 1.0
sine = numpy.sin(2 * numpy.pi * t)

# Ramps (5)
custom_lfos.append(ramp ** 4)
custom_lfos.append(ramp)
custom_lfos.append(triangle)
custom_lfos.append(ramp[::-1])
custom_lfos.append(ramp[::-1] ** 4)

# Bump
window = (1.0 - numpy.cos(4 * numpy.pi * t)) / 2.0
custom_lfos.append(numpy.maximum(sine, 0.0) * window)

custom_lfos.append(square.astype(float))

sine_2 = numpy.sin(2 * numpy.pi * t) + 0.7 * numpy.sin(4 * numpy.pi * t)
sine_3 = numpy.sin(2 * numpy.pi * t) + 0.7 * numpy.sin(6 * numpy.pi * t)
sine_5 = numpy.sin(2 * numpy.pi * t) + 0.7 * numpy.sin(10 * numpy.pi * t)

custom_lfos.append(sine ** 3)
custom_lfos.append(sine)
custom_lfos.append(sine_2)
custom_lfos.append(sine_3)
custom_lfos.append(sine_5)
triangle_3 = numpy.cumsum(numpy.sign(numpy.diff(sine_3)))
custom_lfos.append(numpy.array([0] + list(triangle_3)))

for fold_amount in [1.0, 0.5]:
  rotate = (numpy.arange(0, 256) + 128) % 256
  fold = (1 + fold_amount) * bipolar_triangle[rotate]
  fold[fold > 1.0] = 2.0 - fold[fold > 1.0]
  fold[fold < -1.0] = -2.0 - fold[fold < -1.0]
  custom_lfos.append(-fold)

spike = 2 ** (4 * triangle) - 1
custom_lfos.append(spike)
numpy.random.seed(999)
custom_lfos.append(numpy.random.random((256, 1)).ravel())
custom_lfos.append(sine)

lfo_waveforms = numpy.zeros((257 * len(custom_lfos),), dtype=numpy.uint8)
import pylab
for i, values in enumerate(custom_lfos):
  values = scale(values)
  lfo_waveforms[i * 257: i * 257 + 256] = values
  lfo_waveforms[i * 257 + 256] = values[0]

waveforms = [('lfo_waveforms', lfo_waveforms)]



#!/usr/bin/python2.5
#
# Copyright 2014 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
# See http://creativecommons.org/licenses/MIT/ for more information.
#
# -----------------------------------------------------------------------------
#
# Waveform definitions.

import numpy

WAVETABLE_SIZE = 256

# Waveforms for the trigger shaper ---------------------------------------------

numpy.random.seed(666)

def scale(x):
  if x.min() > 0:
    x = (x - x.min()) / (x.max() - x.min()) * 32767.0
  else:
    abs_max = numpy.abs(x).max()
    x = x / abs_max * 32767.0
  return numpy.round(x)


t = numpy.arange(0, WAVETABLE_SIZE + 1) / float(WAVETABLE_SIZE)

exponential = numpy.exp(-4.0 * t)
ring = numpy.exp(-3.0 * t) * numpy.cos(8.0 * t * numpy.pi)
steps = numpy.sign(numpy.sin(4.0 * t * numpy.pi)) * (2 ** (-numpy.round(t * 2.0)))
noise = numpy.random.randn(WAVETABLE_SIZE + 1, 1).ravel() * ((1 - t) ** 2)

waveforms = [('exponential', scale(exponential))]
waveforms += [('ring', scale(ring))]
waveforms += [('steps', scale(steps))]
waveforms += [('noise', scale(noise))]



"""----------------------------------------------------------------------------
Band-limited waveforms
----------------------------------------------------------------------------"""

WAVETABLE_SIZE = 1024

# Sine wave.
sine = -numpy.sin(numpy.arange(WAVETABLE_SIZE + 1) / float(WAVETABLE_SIZE) * 2 * numpy.pi) * 32767

waveforms.append(('sine', scale(sine)))



#!/usr/bin/python2.5
#
# Copyright 2014 Emilie Gillet.
#
# Author: Emilie Gillet (emilie.o.gillet@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
# See http://creativecommons.org/licenses/MIT/ for more information.
#
# -----------------------------------------------------------------------------
#
# Lookup table definitions.

import numpy as np
import pylab
import scipy.signal

lookup_tables = []

"""----------------------------------------------------------------------------
Sine table.
----------------------------------------------------------------------------"""

size = 1024
t = np.arange(0, size + size / 4 + 1) / float(size) * np.pi * 2
lookup_tables.append(('sin', np.sin(t)))



"""----------------------------------------------------------------------------
XFade table
----------------------------------------------------------------------------"""

size = 257
t = np.arange(0, size) / float(size)
t = 1.04 * t - 0.02
t[t < 0] = 0
t[t >= 1] = 1
t *= np.pi / 2
lookup_tables.append(('xfade_in', np.sin(t) * (2 ** -0.5)))
lookup_tables.append(('xfade_out', np.cos(t) * (2 ** -0.5)))



"""----------------------------------------------------------------------------
Wavefolder LUT.
----------------------------------------------------------------------------"""

WAVETABLE_SIZE = 4096
t = np.arange(0.0, WAVETABLE_SIZE + 1) / WAVETABLE_SIZE

# Waveshape based on Tides
# x = 2 * t - 1
# x = x * 8.0
# x = x / (1 + np.abs(x))
# sine = np.sin(8 * np.pi * x)
# window = np.exp(-x * x * 4) ** 2
# bipolar_fold = sine * window + np.arctan(3 * x) * (1 - window)
# bipolar_fold /= np.abs(bipolar_fold).max()
# bipolar_fold *= 0.8

x = 2 * t - 1
central_bump = np.exp(-x * x * 4) ** 2
x = x * central_bump + (1 - central_bump) * ((t + t ** 1.3) - 1)
x = x * 8.0
x = x / (1 + np.abs(x))

timbre = np.fromfile('warps/resources/timbre.raw', dtype=np.float32)
bipolar_fold = []
for value in x:
  value = (value + 1) * 0.5 * 4095
  value_integral = int(value)
  value_fractional = value - value_integral
  a = timbre[value_integral]
  b = timbre[value_integral + 1]
  bipolar_fold.append(a + (b - a) * value_fractional)
bipolar_fold = np.array(bipolar_fold) * 2.2
lookup_tables += [('bipolar_fold', bipolar_fold)]



"""----------------------------------------------------------------------------
MIDI to normalized frequency table.
----------------------------------------------------------------------------"""

SAMPLE_RATE = 96000

TABLE_SIZE = 256

midi_note = np.arange(0, TABLE_SIZE) - 48
frequency = 440 * 2 ** ((midi_note - 69) / 12.0)
max_frequency = min(12000, SAMPLE_RATE / 2)
frequency[frequency >= max_frequency] = max_frequency
frequency /= SAMPLE_RATE

semitone = 2 ** (np.arange(0, TABLE_SIZE) / 256.0 / 12.0)

lookup_tables.append(('midi_to_f_high', frequency))
lookup_tables.append(('midi_to_f_low', semitone))



"""----------------------------------------------------------------------------
Potentiometer compensation.
----------------------------------------------------------------------------"""

source = [0, 0.006, 0.014, 0.16, 0.33, 0.5, 0.67, 0.84, 0.96, 0.994, 1.00]
dest = [0, 0.1, 0.125, 0.25, 0.375, 0.5, 0.625, 0.750, 0.875, 0.9, 1.00]
n = len(source) - 1
pot_curve = []
for i in xrange(513):
  frac = i / 512.0
  n = 0
  while (frac > source[n + 1]):
    n += 1
  position = (frac - source[n]) / (source[n + 1] - source[n])
  x = dest[n] + position * (dest[n + 1] - dest[n])
  pot_curve += [x]

lookup_tables.append(('pot_curve', pot_curve))



"""----------------------------------------------------------------------------
Allpass filter network for phase shifter.
----------------------------------------------------------------------------"""

BANDWIDTH = 0.495
ORDER = 17
PASSBAND = [10.0, 20000.0]
FFT_SIZE = 65536



def iir_lpf_poles(cutoff, order, warp=True):
  wp = cutoff * np.pi
  ws = np.pi - wp

  # C. Britton Rorabaugh, "Digital Filter Designer's Handbook", p. 94
  k = np.tan(0.5 * wp) / np.tan(0.5 * ws) if warp else wp / ws
  u = 0.5 * (1 - (1 - k ** 2) ** 0.25) / (1 + (1 - k ** 2) ** 0.25)
  q = u + 2 * u ** 5 + 15 * u ** 9 + 150 * u ** 13

  poles = []
  for i in xrange((order - 1) / 2):
    w = (i + 1) * np.pi / order
    num = np.sum(((-1) ** m) * q ** (m * (m + 1)) * np.sin((2 * m + 1) * w) for m in xrange(0, 7))
    den = np.sum(((-1) ** m) * q ** (m * m) * np.cos(2 * m * w) for m in xrange(1, 7))
    l = 2 * q ** 0.25 * num / (1 + 2 * den)
    b = ((1 - k * l ** 2) * (1 - l ** 2 / k)) ** 0.5
    c = 2 * b / (1 + l ** 2)
    poles += [(2 - c) / (2 + c)]
  return poles



def warped_lp_to_ap(lp_poles, band):
  w = np.pi * band
  beta = (np.tan(w[0]) * np.tan(w[1])) ** 0.5
  b = (beta - 1) / (beta + 1)
  ap_poles = []
  # Create array of AP poles and do frequency warping.
  for pole in lp_poles:
    pole = np.sqrt(pole)
    conjugates = [-pole, pole] if pole != 0.0 else [pole]
    ap_poles += [(-c + b) / (-c * b + 1) for c in conjugates]
  return -np.sort(ap_poles)



def phase_shift(x, y):
  xf = np.fft.rfft(x)
  yf = np.fft.rfft(y)
  f = np.arange(0, FFT_SIZE / 2 + 1) / float(FFT_SIZE) * SAMPLE_RATE
  error = np.mod(np.angle(yf / xf) + 2 * np.pi, 2 * np.pi)
  return f, error / np.pi * 180



def iq_decomposition(ap_poles, x=np.eye(FFT_SIZE, 1).ravel()):
  x = [x, x + 0]
  for i, b in enumerate(ap_poles):
    x[i & 1] = scipy.signal.lfilter([-b, 1], [1, -b], x[i & 1])
  return x[0], x[1]



# Coefficients taken from Sean Costello's "hilbert" Csound opcode.
UGSC_FREQS = [0.3609, 1.2524, 2.7412, 5.5671, 11.1573, 22.3423, 44.7581,
              89.6271, 179.6242, 364.7914, 798.4578, 2770.1114]
UGSC_FREQS = np.array(UGSC_FREQS) * 15 * np.pi / SAMPLE_RATE
UGSC_POLES = [(1 - alpha) / (1 + alpha) for alpha in UGSC_FREQS]

# Coefficients computed from the all-pass decomposition of an elliptic halfband
# filter.
AP_POLES = warped_lp_to_ap(
    iir_lpf_poles(BANDWIDTH, ORDER) + [0.0],
    np.array(PASSBAND) / SAMPLE_RATE)

if __name__ == '__main__':
  f, error_1 = phase_shift(*iq_decomposition(UGSC_POLES))
  _, error_2 = phase_shift(*iq_decomposition(AP_POLES))
  pylab.semilogx(f, error_1, 'r')
  pylab.semilogx(f, error_2, 'g')
  pylab.xlim([2, 48000])
  pylab.xlabel('Frequency (Hz)')
  pylab.ylim([90 * 0.99, 90 * 1.01])
  pylab.ylabel('Phase shift (degrees)')
  pylab.show()

lookup_tables.append(('ap_poles', AP_POLES))
